package jmaki.docs;

import org.apache.tools.ant.Task;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.types.FileSet;
import java.util.Vector;
import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;
import java.util.ArrayList;
import java.io.*;
import java.io.File;
import org.json.JSONObject;
import org.json.JSONArray;

/**
 * Parse A Widget Library and Generate HTML documentation based
 * on a widget.json file.
 * 
 * @author Greg Murray
 * @since 2007-01-16
 */
public class GenWidgetDocTask extends Task {

    private StringBuffer pageTemplate = null;
    private StringBuffer libraryTemplate = null;
    private String targetDirectory = "docs";
    private ArrayList targetDirs = new ArrayList();
    private ArrayList filesets = new ArrayList();
    
    /** The message to print. As attribute. */
    String message;
    
    public void setMessage(String msg) {
        message = msg;
    }
    
    public void setTodir(String td) {
        targetDirectory = td;
    }
    
    public void addLibrarybase(String libbase) {
       targetDirs.add(libbase); 
    }
    
    public void addFileSet(FileSet fileset) {
      if (!filesets.contains(fileset)) {
         filesets.add(fileset);
      }
    }    
    
    /** Should the build fail? Defaults to false. As attribute. */
    boolean fail = false;
    public void setFail(boolean b) {
        fail = b;
    }

    /** Support for nested text. */
    public void addText(String text) {
        message = text;
    }

    /** Do the work. */
    public void execute() {
       File targetFile = null;
        if (pageTemplate == null) {
            try {
                FileInputStream fis = new FileInputStream("templates/page.html");
                pageTemplate = loadTemplate(fis); 
            } catch (FileNotFoundException fnfx) {
                System.err.println("GenWidgetDocTask Error: " + fnfx);    
            }                
                
        }
        
       Iterator fit = filesets.iterator();
       while (fit.hasNext()) {
           try {
               FileInputStream fis = new FileInputStream("templates/library.html");
               libraryTemplate = loadTemplate(fis);
           } catch (FileNotFoundException fnfx) {
               System.err.println("GenWidgetDocTask Error: " + fnfx);    
           }
           
           FileSet fs = (FileSet)fit.next();
           targetFile = fs.getDir();
           
           // handle attribute 'message' and nested text
           if (targetFile!=null) {

               ArrayList widgets = new ArrayList();
               StringBuffer pbuff = new StringBuffer();
               try {
                   String base = targetFile.getAbsolutePath();
                   findWidgets(base, targetFile, widgets);
                   Iterator it = widgets.iterator();
                   String packageName = null;
                   while(it != null && it.hasNext()) {
                       Widget w = (Widget)it.next();
                       // this is kind of cheating as we want this passed in via the ant task
                       if (packageName == null) packageName = w.getPackageName();                   
                       pbuff.append("<p><a href=\"" +
                                    w.getPath().replace("\\", "/") +
                                    "/doc.html\">" + w.getShortName() +
                                    "</a></p>");
                       genDocs(w); 
                   }
                   // make packageName 1st char upper case
                   System.out.println("packageName " + packageName);
                   String packageNameBefore = packageName;
                   char c = packageName.charAt(0);
                   packageName = Character.toUpperCase(c) + packageName.substring(1, packageName.length());
                   
                   // write out the library
                   replace( libraryTemplate, "${body}", pbuff.toString());
                   replace( libraryTemplate, "${library}", packageName);  
                   File b = new File(targetDirectory);
                   FileOutputStream fos = new FileOutputStream(new File(b, packageNameBefore + ".html"));
                   writeHTML(new ByteArrayInputStream(libraryTemplate.toString().getBytes()), fos);              
                   
               } catch(Exception e) {
                   System.out.println("Error parsing Widgets: " + e);
                   e.printStackTrace();
               }
               //if (success) log("Success for: " + message);
               // else  log("Failed for: " + message);        
           }
       }
        
        // handle attribute 'fail'
        if (fail) throw new BuildException("Fail requested.");        
    }
    
    class Widget {
        private String name = null;
        private String base;
        private File f;
        private String image = null;
        
        public Widget(String base, File f) {
            this.base = base;
            this.f = f;
        }
        
        public File getFile() {
            return f;   
        }
        
        public void setImage(String image) {
            this.image = image;    
        }
        
        public String getImage() {
            return image;    
        }
        
        public String getName() {
            if (name == null) name =getPath().replace(f.separator, ".");
            return name;
        }
        
        public String getPackageName() {
            int first = getName().indexOf(".");
            if (first != -1) return name.substring(0,first);
            else return name;
        }
        
        
        public String getShortName() {
            int last = getName().lastIndexOf(".");
            if (last != -1) return name.substring(last + 1,name.length());
            else return name;
        }
        
        public String getPath() {
           return f.getAbsolutePath().substring(base.length() + 1,
                                             f.getAbsolutePath().length() - "widget.json".length() -1);
        }
        
        public String toString() {
            return "name=" + getName() + " file=" + f;    
        }  
    }
    
    public void findWidgets(String base, File f , ArrayList widgets) {
        if (f.isDirectory()) {
            String[] children = f.list();
            for (int i=0; i<children.length; i++) {
                findWidgets(base, new File(f, children[i]), widgets);
            }
        } else {
            if ("widget.json".equals(f.getName()) ) {
                Widget w = new Widget(base, f);
                widgets.add(w);
           }
        }
   }
    
    public boolean genDocs(Widget widget) {
        String widgetName = widget.getName();
        System.out.println("Processing " + widgetName);   
    
        try {
            StringBuffer template = new StringBuffer(pageTemplate);
            FileInputStream fis = new FileInputStream(widget.getFile());
            JSONObject jo= loadJSONObject(fis);
            HashMap w = new HashMap();
            if (jo == null) return false;
            // find the general properties
            if (jo.has("name")) {
                 w.put("widgetName", jo.getString("name"));
             }
             if (jo.has("image")) {
                 widget.setImage(jo.getString("image"));
             }
             
             if (jo.has("version")) {
                 w.put("widgetVersion", jo.getString("version"));
             } else {
                 w.put("widgetVersion","");
             }
             
            if (jo.has("author")) {
                 w.put("widgetAuthor", jo.getString("author"));
             } else {
                 w.put("widgetAuthor","");
             }
             
            if (jo.has("url")) {
                 w.put("widgetURL", jo.getString("url"));
             } else {
                 w.put("widgetURL","");
             }             
             if (jo.has("description")) {
                 w.put("widgetDescription", jo.getString("description"));
             }
             // now parse the args
             String argsBody = "";
             if (jo.has("args") ) {
                 argsBody += "<table class='argsTable'>";
                 // header
                 argsBody += "<thead><th>Name</th><th>Description</th><th>Type</th><th>Value Options</th><th>Default Value</th></thead>";
                 JSONArray ja = jo.getJSONArray("args");
                 for (int i =0; i < ja.length(); i++) {
                    argsBody += " <tr>";      
                    JSONObject ai = ja.getJSONObject(i);
                    Iterator it = ai.keys();
                    String ain = null;
                    // make sure the arg has a first key
                    if (it.hasNext()) {
                        ain = (String)it.next();
                    } else {
                        System.err.println("Error parsing args[" + i + "] : No first property for key." ); 
                    }             
                    // get a single argument and it's properties.
                    if (ain != null)    {
                        argsBody += "<td>" + ain + "</td>";
                        JSONObject arg = ai.getJSONObject(ain);
                        
                        if (arg.has("description")) {
                            argsBody += "<td>" + arg.getString("description") + "</td>";
                        } else {
                             argsBody += "<td></td>";
                        }    
                        
                        if (arg.has("type")) {
                            argsBody += "<td>" + arg.getString("type") + "</td>";
                        } else {
                             argsBody += "<td></td>";
                        }                  
                        // get possible values - only the non-readable
                        if (arg.has("values")) {
                            JSONArray vals = arg.getJSONArray("values");
                            String valString = "";
                            for (int vi=0; vi < vals.length(); vi ++) {
                                JSONObject ivals = vals.getJSONObject(vi);
                                if (ivals.length() > 0) {
                                    String ext = "";
                                    if (ivals.has("description")) {
                                        ext = " - " +   ivals.getString("description");
                                    }
                                    valString += ivals.getString("value") + ext;
                                }
                                if (vi < vals.length() - 1) valString += "<br>";
                            }
                            argsBody += "<td>" + valString + "</td>";
                        } else {
                             argsBody += "<td></td>";
                        }                                            
                        if (arg.has("defaultValue")) {
                            argsBody += "<td>" + arg.getString("defaultValue") + "</td>";
                        } else {
                             argsBody += "<td></td>";
                        }
                    }
                    argsBody += " </tr>"; 
                 }
                 argsBody += "</table>";
             } else {
                argsBody = "N/A";    
             }
             w.put("argsBody", argsBody);
             String vb = "";
              if (jo.has("value") ) {
                  JSONObject valo = jo.getJSONObject("value");
                 vb += "<table class='valueTable'>";
                 // header
                 vb += "<thead><th>Required</th><th>Type</th><th>Default Value</th></thead>";
                 vb += "<tr>";
                 if (valo.has("required")) {
                    vb += "<td>" + valo.getString("required") + "</td>"; 
                 } else {
                    vb += "<td></td>";
                 }
                 if (valo.has("type")) {
                    vb += "<td>" + valo.getString("type") + "</td>"; 
                 } else {
                    vb += "<td></td>";
                 }
                 if (valo.has("defaultValue")) {
                    vb += "<td>" + valo.getString("defaultValue") + "</td>"; 
                 } else {
                    vb += "<td></td>";
                 }
                  vb += "</tr></table>";         
             } else {
                 vb = "N/A";
             }
             w.put("valueBody", vb);
             w.put("widgetId", widget.getName());
             w.put("widgetPackage", widget.getPackageName());
             
             // now generate the document
             File dir = new File(targetDirectory +  File.separator + widget.getPath());
             boolean s = dir.mkdirs();
             if (widget.getImage() != null) {
                 // copy the graphic
                try {
                     File img = new File(widget.getFile().getParent(),  widget.getImage());
                     File imgo = new File(dir, "widget.jpg");
                     imgo.createNewFile();
                     if (imgo.exists()) {
                         FileInputStream fis2  = new FileInputStream(img);
                         FileOutputStream fos = new FileOutputStream(imgo);
        			     byte[] buffer = new byte[1024];
        			     int read = 0;
                         while (true) {
                            read = fis2.read(buffer);
                            if (read <= 0) break;
                                fos.write(buffer, 0, read );
                        }
                        fis2.close();
                        fos.close();
                        w.put("widgetImage", "widget.jpg");
                    }
                } catch (java.io.IOException iox) {
                    System.err.println("GenWidgetDocTask Error: " + iox); 
                     w.put("widgetImage", "");
                }
             } 
             
             // now apply the properies to the template
             Iterator it = w.keySet().iterator();
             while (it.hasNext()) {
                 String key = (String)it.next();
                 replace( template, "${" + key + "}", (String)w.get(key));   
             }
             
             // create the doc
             FileOutputStream fos = new FileOutputStream(new File(dir, "doc.html"));
             writeHTML(new ByteArrayInputStream(template.toString().getBytes()), fos);
            return true;
        } catch (org.json.JSONException jex) {
            System.err.println("GenWidgetDocTask JSON Error: " + jex);
            jex.printStackTrace();
        } catch (FileNotFoundException fnfx) {
            System.err.println("GenWidgetDocTask Error: " + fnfx);    
        } catch (java.io.IOException iox) {
            System.err.println("GenWidgetDocTask Error: " + iox);    
        }
        return false;
    }
    
    /** Store nested 'message's. */
    Vector messages = new Vector();

    /** Factory method for creating nested 'message's. */
    public Message createMessage() {
        Message msg = new Message();
        messages.add(msg);
        return msg;
    }

    /** A nested 'message'. */
    public class Message {
        // Bean constructor
        public Message() {}

        /** Message to print. */
        String msg;
        public void setMsg(String msg) { this.msg = msg; }
        public String getMsg() { return msg; }
    }
    
   public static JSONObject loadJSONObject(InputStream in) {
        ByteArrayOutputStream out = null;
        try {
			byte[] buffer = new byte[1024];
			int read = 0;
            out = new ByteArrayOutputStream();
            while (true) {
                read = in.read(buffer);
                if (read <= 0) break;
                out.write(buffer, 0, read );
            }
            return new JSONObject(out.toString());
        } catch (Exception e) {
           System.out.println("GenWidgetDocTask: error reading in json "  + e);
        } finally {
            try {
                if (in != null) {
                    in.close();
                }
                if (out != null) {
                    out.flush();
                    out.close();
                }
            } catch (Exception e) {
            }
        }
        return null;
    }
    

    public static StringBuffer loadTemplate(InputStream in) {
        ByteArrayOutputStream out = null;
        try {
			byte[] buffer = new byte[1024];
			int read = 0;
            out = new ByteArrayOutputStream();
            while (true) {
                read = in.read(buffer);
                if (read <= 0) break;
                out.write(buffer, 0, read );
            }
            return new StringBuffer(out.toString("UTF-8"));
        } catch (Exception e) {
           System.out.println("GenWidgetDocTask: error reading in json "  + e);
        } finally {
            try {
                if (in != null) {
                    in.close();
                }
                if (out != null) {
                    out.flush();
                    out.close();
                }
            } catch (Exception e) {
            }
        }
        return null;
    }
    
    public static void replace(StringBuffer buff, String target, String replacement) {
        if (buff == null || target == null || replacement == null) {
          return;
        }
        int index = 0;
        while (index < buff.length()) {
            index = buff.indexOf(target);
            if (index == -1) {
                break;
            }
            buff.replace(index, index +  target.length(), replacement);
            index += replacement.length() + 1;
        }
    }
    
    private void writeHTML(InputStream in, OutputStream out) {
        byte[] bytes = new byte[1024];
        try {
          int read = 0;
          while ((read = in.read(bytes)) > 0) {
               out.write(bytes, 0, read);
          }
          in.close();
          out.close();
        } catch (IOException e) {
            System.out.println("GenWidgetDocTask:read from stream error:"  + e);
        }
    }

}