/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 *
 * You may not modify, use, reproduce, or distribute this software except in
 * compliance with the terms of the License at:
 * http://developer.sun.com/berkeley_license.html
 */

package com.sun.jmaki.faces.component;

import javax.faces.component.UIComponentBase;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;

/**
 * <p>Component wrapper for the Dojo Clock widget.</p>
 */
public class DojoClock extends UIComponentBase {


    // ------------------------------------------------------------ Constructors


    public DojoClock() {
        super();
        setRendererType("com.sun.jmaki.faces.DojoClock");
    }


    // ------------------------------------------------------ Instance Variables


    private String clockType;
    private String handBorderColor;
    private String handColor;
    private String label;
    private String labelColor;
    private String style;
    private String styleClass;
    private int timeZoneOffset;
    private boolean timeZoneOffset_set;
    private String topLabelColor;


    // ----------------------------------------------------- UIComponent Methods


    public String getFamily() {
        return "com.sun.jmaki.faces.DojoClock";
    }


    public void restoreState(FacesContext context, Object state) {
        Object[] values = (Object[]) state;
        super.restoreState(context, values[0]);
        clockType = (String) values[1];
        handBorderColor = (String) values[2];
        handColor = (String) values[3];
        label = (String) values[4];
        labelColor = (String) values[5];
        style = (String) values[6];
        styleClass = (String) values[7];
        timeZoneOffset = ((Integer) values[8]).intValue();
        timeZoneOffset_set = ((Boolean) values[9]).booleanValue();
        topLabelColor = (String) values[10];
    }


    public Object saveState(FacesContext context) {
        Object[] values = new Object[11];
        values[0] = super.saveState(context);
        values[1] = clockType;
        values[2] = handBorderColor;
        values[3] = handColor;
        values[4] = label;
        values[5] = labelColor;
        values[6] = style;
        values[7] = styleClass;
        values[8] = new Integer(timeZoneOffset);
        values[9] = timeZoneOffset_set ? Boolean.TRUE : Boolean.FALSE;
        values[10] = topLabelColor;
        return values;
    }


    // ------------------------------------------------------- Public Properties

    public String getClockType() {
        if (this.clockType != null) {
            return this.clockType;
        }
        ValueBinding vb = getValueBinding("clockType");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setClockType(String clockType) {
        this.clockType = clockType;
    }

    public String getHandBorderColor() {
        if (this.handBorderColor != null) {
            return this.handBorderColor;
        }
        ValueBinding vb = getValueBinding("handBorderColor");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setHandBorderColor(String handBorderColor) {
        this.handBorderColor = handBorderColor;
    }

    public String getHandColor() {
        if (this.handColor != null) {
            return this.handColor;
        }
        ValueBinding vb = getValueBinding("handColor");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setHandColor(String handColor) {
        this.handColor = handColor;
    }

    public String getLabel() {
        if (this.label != null) {
            return this.label;
        }
        ValueBinding vb = getValueBinding("label");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setLabel(String label) {
        this.label = label;
    }

    public String getLabelColor() {
        if (this.labelColor != null) {
            return this.labelColor;
        }
        ValueBinding vb = getValueBinding("labelColor");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setLabelColor(String labelColor) {
        this.labelColor = labelColor;
    }

    public String getStyle() {
        if (this.style != null) {
            return this.style;
        }
        ValueBinding vb = getValueBinding("style");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setStyle(String style) {
        this.style = style;
    }

    public String getStyleClass() {
        if (this.styleClass != null) {
            return this.styleClass;
        }
        ValueBinding vb = getValueBinding("styleClass");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setStyleClass(String styleClass) {
        this.styleClass = styleClass;
    }

    public int getTimeZoneOffset() {
        if (this.timeZoneOffset_set) {
            return this.timeZoneOffset;
        }
        ValueBinding vb = getValueBinding("timeZoneOffset");
        if (vb != null) {
            return ((Integer) vb.getValue(getFacesContext())).intValue();
        }
        return this.timeZoneOffset;
    }

    public void setTimeZoneOffset(int timeZoneOffset) {
        this.timeZoneOffset = timeZoneOffset;
        this.timeZoneOffset_set = true;
    }

    public String getTopLabelColor() {
        if (this.topLabelColor != null) {
            return this.topLabelColor;
        }
        ValueBinding vb = getValueBinding("topLabelColor");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setTopLabelColor(String topLabelColor) {
        this.topLabelColor = topLabelColor;
    }


}
