/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 *
 * You may not modify, use, reproduce, or distribute this software except in
 * compliance with the terms of the License at:
 * http://developer.sun.com/berkeley_license.html
 */

package com.sun.jmaki.faces.component;

import javax.faces.component.UIInput;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;

/**
 * <p>Component wrapper for the Dojo Editor widget.</p>
 */
public class DojoEditor extends UIInput {


    // ------------------------------------------------------------ Constructors


    public DojoEditor() {
        super();
        setRendererType("com.sun.jmaki.faces.DojoEditor");
    }


    // ------------------------------------------------------ Instance Variables


    private String style;
    private String styleClass;
    private String toolbarType;


    // ----------------------------------------------------- UIComponent Methods


    public String getFamily() {
        return "com.sun.jmaki.faces.DojoEditor";
    }


    public void restoreState(FacesContext context, Object state) {
        Object[] values = (Object[]) state;
        super.restoreState(context, values[0]);
        style = (String) values[1];
        styleClass = (String) values[2];
        toolbarType = (String) values[3];
    }


    public Object saveState(FacesContext context) {
        Object[] values = new Object[4];
        values[0] = super.saveState(context);
        values[1] = style;
        values[2] = styleClass;
        values[3] = toolbarType;
        return values;
    }


    // ------------------------------------------------------- Public Properties

    public String getStyle() {
        if (this.style != null) {
            return this.style;
        }
        ValueBinding vb = getValueBinding("style");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setStyle(String style) {
        this.style = style;
    }

    public String getStyleClass() {
        if (this.styleClass != null) {
            return this.styleClass;
        }
        ValueBinding vb = getValueBinding("styleClass");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setStyleClass(String styleClass) {
        this.styleClass = styleClass;
    }

    public String getToolbarType() {
        if (this.toolbarType != null) {
            return this.toolbarType;
        }
        ValueBinding vb = getValueBinding("toolbarType");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setToolbarType(String toolbarType) {
        this.toolbarType = toolbarType;
    }

}
