/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 *
 * You may not modify, use, reproduce, or distribute this software except in
 * compliance with the terms of the License at:
 * http://developer.sun.com/berkeley_license.html
 */

package com.sun.jmaki.faces.component;

import javax.faces.component.UIOutput;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;

/**
 * <p>Component wrapper for the Flickr Word widget.</p>
 */
public class FlickrWord extends UIOutput {


    // ------------------------------------------------------------ Constructors


    public FlickrWord() {
        super();
        setRendererType("com.sun.jmaki.faces.FlickrWord");
    }


    // ------------------------------------------------------ Instance Variables


    private String apiKey;
    // "converter" is inherited
    private int size;
    private boolean size_set = false;
    private String style;
    private String styleClass;
    private boolean unique;
    private boolean unique_set = false;
    // "value" is inherited


    // ----------------------------------------------------- UIComponent Methods


    public String getFamily() {
        return "com.sun.jmaki.faces.FlickrWord";
    }


    public void restoreState(FacesContext context, Object state) {
        Object[] values = (Object[]) state;
        super.restoreState(context, values[0]);
        apiKey = (String) values[1];
        size = ((Integer) values[2]).intValue();
        size_set = ((Boolean) values[3]).booleanValue();
        style = (String) values[4];
        styleClass = (String) values[5];
        unique = ((Boolean) values[6]).booleanValue();
        unique_set = ((Boolean) values[7]).booleanValue();
    }


    public Object saveState(FacesContext context) {
        Object[] values = new Object[8];
        values[0] = super.saveState(context);
        values[1] = apiKey;
        values[2] = new Integer(size);
        values[3] = size_set ? Boolean.TRUE : Boolean.FALSE;
        values[4] = style;
        values[5] = styleClass;
        values[6] = unique ? Boolean.TRUE : Boolean.FALSE;
        values[7] = unique_set ? Boolean.TRUE : Boolean.FALSE;
        return values;
    }


    // ------------------------------------------------------- Public Properties

    public String getApiKey() {
        if (this.apiKey != null) {
            return this.apiKey;
        }
        ValueBinding vb = getValueBinding("apiKey");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setApiKey(String apiKey) {
        this.apiKey = apiKey;
    }

    public int getSize() {
        if (this.size_set) {
            return this.size;
        }
        ValueBinding vb = getValueBinding("size");
        if (vb != null) {
            return ((Integer) vb.getValue(getFacesContext())).intValue();
        }
        return this.size;
    }

    public void setSize(int size) {
        this.size = size;
        this.size_set = true;
    }

    public String getStyle() {
        if (this.style != null) {
            return this.style;
        }
        ValueBinding vb = getValueBinding("style");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setStyle(String style) {
        this.style = style;
    }

    public String getStyleClass() {
        if (this.styleClass != null) {
            return this.styleClass;
        }
        ValueBinding vb = getValueBinding("styleClass");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setStyleClass(String styleClass) {
        this.styleClass = styleClass;
    }

    public boolean isUnique() {
        if (this.unique_set) {
            return this.unique;
        }
        ValueBinding vb = getValueBinding("unique");
        if (vb != null) {
            return ((Boolean) vb.getValue(getFacesContext())).booleanValue();
        }
        return this.unique;
    }

    public void setUnique(boolean unique) {
        this.unique = unique;
        this.unique_set = true;
    }

}
