/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 *
 * You may not modify, use, reproduce, or distribute this software except in
 * compliance with the terms of the License at:
 * http://developer.sun.com/berkeley_license.html
 */

package com.sun.jmaki.faces.component;

import javax.faces.component.UIComponentBase;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;

/**
 * <p>Component wrapper for the Spry Accordion widget.</p>
 */
public class SpryAccordion extends UIComponentBase {


    // ------------------------------------------------------------ Constructors


    public SpryAccordion() {
        super();
        setRendererType("com.sun.jmaki.faces.SpryAccordion");
    }


    // ------------------------------------------------------ Instance Variables


    private int current;
    private boolean current_set;
    private String gradient;
    private String style;
    private String styleClass;


    // ----------------------------------------------------- UIComponent Methods


    public String getFamily() {
        return "com.sun.jmaki.faces.SpryAccordion";
    }


    public boolean getRendersChildren() {
        return true;
    }


    public void restoreState(FacesContext context, Object state) {
        Object[] values = (Object[]) state;
        super.restoreState(context, values[0]);
        current = ((Integer) values[1]).intValue();
        current_set = ((Boolean) values[2]).booleanValue();
        gradient = (String) values[3];
        style = (String) values[4];
        styleClass = (String) values[5];
    }


    public Object saveState(FacesContext context) {
        Object[] values = new Object[6];
        values[0] = super.saveState(context);
        values[1] = new Integer(current);
        values[2] = current_set ? Boolean.TRUE : Boolean.FALSE;
        values[3] = gradient;
        values[4] = style;
        values[5] = styleClass;
        return values;
    }


    // ------------------------------------------------------- Public Properties

    public int getCurrent() {
        if (this.current_set) {
            return this.current;
        }
        ValueBinding vb = getValueBinding("current");
        if (vb != null) {
            return (Integer) vb.getValue(getFacesContext());
        }
        return this.current;
    }

    public void setCurrent(int current) {
        this.current = current;
        this.current_set = true;
    }

    public String getGradient() {
        if (this.gradient != null) {
            return this.gradient;
        }
        ValueBinding vb = getValueBinding("gradient");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setGradient(String gradient) {
        this.gradient = gradient;
    }

    public String getStyle() {
        if (this.style != null) {
            return this.style;
        }
        ValueBinding vb = getValueBinding("style");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setStyle(String style) {
        this.style = style;
    }

    public String getStyleClass() {
        if (this.styleClass != null) {
            return this.styleClass;
        }
        ValueBinding vb = getValueBinding("styleClass");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setStyleClass(String styleClass) {
        this.styleClass = styleClass;
    }


}
