/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 *
 * You may not modify, use, reproduce, or distribute this software except in
 * compliance with the terms of the License at:
 * http://developer.sun.com/berkeley_license.html
 */

package com.sun.jmaki.faces.component;

import javax.faces.component.UIOutput;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;

/**
 * <p>Component wrapper for the Yahoo Tabbed View sub-widget.</p>
 */
public class YahooTabbedViewTab extends UIOutput {


    // ------------------------------------------------------------ Constructors


    public YahooTabbedViewTab() {
        super();
        setRendererType(null);
    }


    // ------------------------------------------------------ Instance Variables


    // "converter" is inherited
    private boolean disabled = false;
    private boolean disabled_set = false;
    private String style;
    private String styleClass;
    // "value" is inherited


    // ----------------------------------------------------- UIComponent Methods


    public String getFamily() {
        return "com.sun.jmaki.faces.YahooTabbedViewTab";
    }


    public boolean getRendersChildren() {
        return true;
    }


    public void restoreState(FacesContext context, Object state) {
        Object[] values = (Object[]) state;
        super.restoreState(context, values[0]);
        disabled = ((Boolean) values[1]).booleanValue();
        disabled_set = ((Boolean) values[2]).booleanValue();
        style = (String) values[3];
        styleClass = (String) values[4];
    }


    public Object saveState(FacesContext context) {
        Object[] values = new Object[5];
        values[0] = super.saveState(context);
        values[1] = disabled ? Boolean.TRUE : Boolean.FALSE;
        values[2] = disabled_set ? Boolean.TRUE : Boolean.FALSE;
        values[3] = style;
        values[4] = styleClass;
        return values;
    }


    // ------------------------------------------------------- Public Properties


    public boolean isDisabled() {
        if (this.disabled_set) {
            return this.disabled;
        }
        ValueBinding vb = getValueBinding("disabled");
        if (vb != null) {
            return ((Boolean) vb.getValue(getFacesContext())).booleanValue();
        }
        return false;
    }

    public void setDisabled(boolean disabled) {
        this.disabled = disabled;
        this.disabled_set = true;
    }

    public String getStyle() {
        if (this.style != null) {
            return this.style;
        }
        ValueBinding vb = getValueBinding("style");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setStyle(String style) {
        this.style = style;
    }

    public String getStyleClass() {
        if (this.styleClass != null) {
            return this.styleClass;
        }
        ValueBinding vb = getValueBinding("styleClass");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setStyleClass(String styleClass) {
        this.styleClass = styleClass;
    }

}
