/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 *
 * You may not modify, use, reproduce, or distribute this software except in
 * compliance with the terms of the License at:
 * http://developer.sun.com/berkeley_license.html
 */

package com.sun.jmaki.faces.component;

import javax.faces.component.UIOutput;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;

/**
 * <p>Component wrapper for the Yahoo Tree View widget.</p>
 */
public class YahooTreeView extends UIOutput {


    // ------------------------------------------------------------ Constructors


    public YahooTreeView() {
        super();
        setRendererType("com.sun.jmaki.faces.YahooTreeView");
    }


    // ------------------------------------------------------ Instance Variables


    private String collapseAnim;
    // "converter" is inherited
    private String expandAnim;
    private boolean expanded = false;
    private boolean expanded_set = false;
    private String style;
    private String styleClass;
    // "value" is inherited


    // ----------------------------------------------------- UIComponent Methods


    public String getFamily() {
        return "com.sun.jmaki.faces.YahooTreeView";
    }


    public boolean getRendersChildren() {
        return true;
    }


    public void restoreState(FacesContext context, Object state) {
        Object[] values = (Object[]) state;
        super.restoreState(context, values[0]);
        collapseAnim = (String) values[1];
        expandAnim = (String) values[2];
        expanded = ((Boolean) values[3]).booleanValue();
        expanded_set = ((Boolean) values[4]).booleanValue();
        style = (String) values[5];
        styleClass = (String) values[6];
    }


    public Object saveState(FacesContext context) {
        Object[] values = new Object[7];
        values[0] = super.saveState(context);
        values[1] = collapseAnim;
        values[2] = expandAnim;
        values[3] = this.expanded ? Boolean.TRUE : Boolean.FALSE;
        values[4] = this.expanded_set ? Boolean.TRUE : Boolean.FALSE;
        values[5] = style;
        values[6] = styleClass;
        return values;
    }


    // ------------------------------------------------------- Public Properties

    public String getCollapseAnim() {
        if (this.collapseAnim != null) {
            return this.collapseAnim;
        }
        ValueBinding vb = getValueBinding("collapseAnim");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setCollapseAnim(String collapseAnim) {
        this.collapseAnim = collapseAnim;
    }

    public String getExpandAnim() {
        if (this.expandAnim != null) {
            return this.expandAnim;
        }
        ValueBinding vb = getValueBinding("expandAnim");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setExpandAnim(String expandAnim) {
        this.expandAnim = expandAnim;
    }

    public boolean isExpanded() {
        if (this.expanded_set) {
            return this.expanded;
        }
        ValueBinding vb = getValueBinding("expanded");
        if (vb != null) {
            return ((Boolean) vb.getValue(getFacesContext())).booleanValue();
        }
        return false;
    }

    public void setExpanded(boolean expanded) {
        this.expanded = expanded;
        this.expanded_set = true;
    }

    public String getStyle() {
        if (this.style != null) {
            return this.style;
        }
        ValueBinding vb = getValueBinding("style");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setStyle(String style) {
        this.style = style;
    }

    public String getStyleClass() {
        if (this.styleClass != null) {
            return this.styleClass;
        }
        ValueBinding vb = getValueBinding("styleClass");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setStyleClass(String styleClass) {
        this.styleClass = styleClass;
    }


}
