/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 *
 * You may not modify, use, reproduce, or distribute this software except in
 * compliance with the terms of the License at:
 * http://developer.sun.com/berkeley_license.html
 */

package com.sun.jmaki.faces.component;

import javax.faces.component.UIOutput;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;

/**
 * <p>Component wrapper for the Yahoo Tree View Node sub-widget.</p>
 */
public class YahooTreeViewNode extends UIOutput {


    // ------------------------------------------------------------ Constructors


    public YahooTreeViewNode() {
        super();
        setRendererType(null);
    }


    // ------------------------------------------------------ Instance Variables


    private String contentStyle = "ygtvhtml";
    // "converter" is inherited
    private boolean expanded = false;
    private boolean expanded_set = false;
    private String href;
    private String labelStyle = "ygtvlabel";
    private boolean multiExpand = true;
    private boolean multiExpand_set = false;
    private String style;
    private String styleClass;
    private String target;
    // "value" is inherited


    // ----------------------------------------------------- UIComponent Methods


    public String getFamily() {
        return "com.sun.jmaki.faces.YahooTreeViewNode";
    }


    public boolean getRendersChildren() {
        return true;
    }


    public void restoreState(FacesContext context, Object state) {
        Object[] values = (Object[]) state;
        super.restoreState(context, values[0]);
        contentStyle = (String) values[1];
        expanded = ((Boolean) values[2]).booleanValue();
        expanded_set = ((Boolean) values[3]).booleanValue();
        href = (String) values[4];
        labelStyle = (String) values[5];
        multiExpand = ((Boolean) values[6]).booleanValue();
        multiExpand_set = ((Boolean) values[7]).booleanValue();
        style = (String) values[8];
        styleClass = (String) values[9];
        target = (String) values[10];
    }


    public Object saveState(FacesContext context) {
        Object[] values = new Object[11];
        values[0] = super.saveState(context);
        values[1] = contentStyle;
        values[2] = expanded ? Boolean.TRUE : Boolean.FALSE;
        values[3] = expanded_set ? Boolean.TRUE : Boolean.FALSE;
        values[4] = href;
        values[5] = labelStyle;
        values[6] = multiExpand ? Boolean.TRUE : Boolean.FALSE;
        values[7] = multiExpand_set ? Boolean.TRUE : Boolean.FALSE;
        values[8] = style;
        values[9] = styleClass;
        values[10] = target;
        return values;
    }


    // ------------------------------------------------------- Public Properties

    public String getContentStyle() {
        if (this.contentStyle != null) {
            return this.contentStyle;
        }
        ValueBinding vb = getValueBinding("contentStyle");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setContentStyle(String contentStyle) {
        this.contentStyle = contentStyle;
    }

    public boolean isExpanded() {
        if (this.expanded_set) {
            return this.expanded;
        }
        ValueBinding vb = getValueBinding("expanded");
        if (vb != null) {
            return ((Boolean) vb.getValue(getFacesContext())).booleanValue();
        }
        return false;
    }

    public void setExpanded(boolean expanded) {
        this.expanded = expanded;
        this.expanded_set = true;
    }

    public String getHref() {
        if (this.href != null) {
            return this.href;
        }
        ValueBinding vb = getValueBinding("href");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setHref(String href) {
        this.href = href;
    }

    public String getLabelStyle() {
        if (this.labelStyle != null) {
            return this.labelStyle;
        }
        ValueBinding vb = getValueBinding("labelStyle");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setLabelStyle(String labelStyle) {
        this.labelStyle = labelStyle;
    }

    public boolean isMultiExpand() {
        if (this.multiExpand_set) {
            return this.multiExpand;
        }
        ValueBinding vb = getValueBinding("multiExpand");
        if (vb != null) {
            return ((Boolean) vb.getValue(getFacesContext())).booleanValue();
        }
        return false;
    }

    public void setMultiExpand(boolean multiExpand) {
        this.multiExpand = multiExpand;
        this.multiExpand_set = true;
    }

    public String getStyle() {
        if (this.style != null) {
            return this.style;
        }
        ValueBinding vb = getValueBinding("style");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setStyle(String style) {
        this.style = style;
    }

    public String getStyleClass() {
        if (this.styleClass != null) {
            return this.styleClass;
        }
        ValueBinding vb = getValueBinding("styleClass");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setStyleClass(String styleClass) {
        this.styleClass = styleClass;
    }

    public String getTarget() {
        if (this.target != null) {
            return this.target;
        }
        ValueBinding vb = getValueBinding("target");
        if (vb != null) {
            return (String) vb.getValue(getFacesContext());
        }
        return null;
    }

    public void setTarget(String target) {
        this.target = target;
    }

}
