/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 *
 * You may not modify, use, reproduce, or distribute this software except in
 * compliance with the terms of the License at:
 * http://developer.sun.com/berkeley_license.html
 */

package com.sun.jmaki.faces.renderer;

import com.sun.jmaki.faces.component.DojoClock;
import java.beans.Beans;
import java.io.IOException;
import java.net.URL;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

/**
 * <p>Renderer for the Dojo Clock widget.</p>
 */
public final class DojoClockRenderer extends AbstractRenderer {


    // ------------------------------------------------------ Manifest Constants


    // -------------------------------------------------------- Renderer Methods


    // ------------------------------------------------ AbstractRenderer Methods


    /**
     * <p>Render the inner design time representation of this widget.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     * @param writer <code>ResponseWriter</code> to write to
     *
     * @exception IOException if an input/output error occurs
     */
    protected void designEnd(FacesContext context, UIComponent component,
                             ResponseWriter writer) throws IOException {

        writer.startElement("img", component);
        URL url = this.getClass().getResource
                ("/com/sun/jmaki/faces/resources/dojo/dojo-clock.jpg");
        writer.writeURIAttribute("src", url.toExternalForm(), null);
        writer.endElement("img");

    }


    /**
     * <p>Return an object literal containing the <code>args</code>
     * parameter to be transmitted to the client.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     */
    protected String paramArgs(FacesContext context, UIComponent component) {

        DojoClock widget = (DojoClock) component;
        ObjectLiteral ol = new ObjectLiteral();
        ol.add("clockType", widget.getClockType());
        if (widget.getTimeZoneOffset() != 0) {
            ol.add("timeZoneOffset", widget.getTimeZoneOffset());
        }
        ol.add("label", widget.getLabel());
        ol.add("labelColor", widget.getLabelColor());
        ol.add("topLabelColor", widget.getTopLabelColor());
        ol.add("handColor", widget.getHandColor());
        ol.add("handBorderColor", widget.getHandBorderColor());
        // FIXME - component does not support secondHandColor
        return ol.toString();

    }


    /**
     * <p>Return the <code>name</code> value for this widget.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     */
    protected String paramName(FacesContext context, UIComponent component) {

        return "dojo.clock";

    }


    // --------------------------------------------------------- Private Methods


}
