/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 *
 * You may not modify, use, reproduce, or distribute this software except in
 * compliance with the terms of the License at:
 * http://developer.sun.com/berkeley_license.html
 */

package com.sun.jmaki.faces.renderer;

import com.sun.jmaki.faces.component.DojoEditor;
import java.beans.Beans;
import java.io.IOException;
import java.net.URL;
import javax.faces.component.UIComponent;
import javax.faces.component.ValueHolder;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

/**
 * <p>Renderer for the Dojo Editor widget.</p>
 */
public final class DojoEditorRenderer extends AbstractRenderer {


    // ------------------------------------------------------ Manifest Constants


    // -------------------------------------------------------- Renderer Methods


    // ------------------------------------------------ AbstractRenderer Methods


    /**
     * <p>Render the inner design time representation of this widget.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     * @param writer <code>ResponseWriter</code> to write to
     *
     * @exception IOException if an input/output error occurs
     */
    protected void designEnd(FacesContext context, UIComponent component,
                             ResponseWriter writer) throws IOException {

        writer.startElement("table", component);
        writer.writeAttribute("style", "width: 100%", null);
        writer.write("\n");

        writer.startElement("tr", component);
        writer.startElement("td", component);
        writer.writeAttribute("style", "background-color: yellow; text-align: center", null);
        writer.writeText("Rich Text Editor", null);
        writer.endElement("td");
        writer.endElement("tr");
        writer.write("\n");

        writer.startElement("tr", component);
        writer.startElement("td", component);
        writer.startElement("textarea", component);
        writer.writeAttribute("id", component.getClientId(context) + "_value", "id");
        writer.writeAttribute("style", "height: 100%; width: 100%", null);
        writer.writeText(valueString(context, component), "value");
        writer.endElement("textarea");
        writer.endElement("td");
        writer.endElement("tr");
        writer.write("\n");

        writer.endElement("table");
        writer.write("\n");

    }


    /**
     * <p>Return an object literal containing the <code>args</code>
     * parameter to be transmitted to the client.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     */
    protected String paramArgs(FacesContext context, UIComponent component) {

        DojoEditor widget = (DojoEditor) component;
        ObjectLiteral ol = new ObjectLiteral();
        ol.add("toolbar", widget.getToolbarType());
        return ol.toString();

    }


    /**
     * <p>Return the <code>name</code> value for this widget.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     */
    protected String paramName(FacesContext context, UIComponent component) {

        return "dojo.editor";

    }


    /**
     * <p>Return the current value of the editor widget, which represents
     * the initial value to be edited.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     */
    protected String paramValue(FacesContext context, UIComponent component) {

        return valueString(context, component);

    }


    // --------------------------------------------------------- Private Methods


}
