/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 *
 * You may not modify, use, reproduce, or distribute this software except in
 * compliance with the terms of the License at:
 * http://developer.sun.com/berkeley_license.html
 */

package com.sun.jmaki.faces.renderer;

import com.sun.jmaki.faces.component.FlickrWord;
import java.beans.Beans;
import java.io.IOException;
import java.net.URL;
import javax.faces.component.UIComponent;
import javax.faces.component.ValueHolder;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;

/**
 * <p>Renderer for the Flickr Word widget.</p>
 */
public final class FlickrWordRenderer extends AbstractRenderer {


    // ------------------------------------------------------ Manifest Constants


    // -------------------------------------------------------- Renderer Methods


    // ------------------------------------------------ AbstractRenderer Methods


    /**
     * <p>Render the inner design time representation of this widget.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     * @param writer <code>ResponseWriter</code> to write to
     *
     * @exception IOException if an input/output error occurs
     */
    protected void designEnd(FacesContext context, UIComponent component,
                             ResponseWriter writer) throws IOException {

        writer.startElement("span", component);
        writer.writeAttribute("style", "font-size: " + ((FlickrWord) component).getSize() + "pt", null);
        writer.writeText(valueString(context, component), "value");
        writer.endElement("span");

    }


    /**
     * <p>Return an object literal containing the <code>args</code>
     * parameter to be transmitted to the client.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     */
    protected String paramArgs(FacesContext context, UIComponent component) {

        FlickrWord widget = (FlickrWord) component;
        ObjectLiteral ol = new ObjectLiteral();
        ol.add("word", valueString(context, component));
        ol.add("repeatCharacters", !widget.isUnique());
        ol.add("size", widget.getSize());
        ol.add("apikey", widget.getApiKey());
        return ol.toString();

    }


    /**
     * <p>Return the <code>name</code> value for this widget.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     */
    protected String paramName(FacesContext context, UIComponent component) {

        return "flickr.word";

    }


    /**
     * <p>Return <code>null</code> because the relevant value is already
     * included in the <code>word</code> attribute of the value returned
     * by <code>paramArgs()</code>.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param component <code>UIComponent</code> being rendered
     */
    protected String paramValue(FacesContext context, UIComponent component) {

        return null;

    }


    // --------------------------------------------------------- Private Methods


}
