/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 *
 * You may not modify, use, reproduce, or distribute this software except in
 * compliance with the terms of the License at:
 * http://developer.sun.com/berkeley_license.html
 */

package com.sun.jmaki.faces.tag;

import javax.faces.component.UIComponent;
import javax.faces.el.ValueBinding;

/**
 * <p>Abstract base class for EditableValueHolder component tags.</p>
 */
public abstract class AbstractEditableValueHolderComponentTag extends AbstractValueHolderComponentTag {
    

    // ------------------------------------------------------ Instance Variables


    private String immediate;
    private String required;
    private String validator;


    // -------------------------------------------------- UIComponentTag Methods


    /**
     * <p>Return the component type required for this tag.</p>
     */
    public abstract String getComponentType();


    /**
     * <p>Return the renderer type required for this tag.</p>
     */
    public abstract String getRendererType();


    /**
     * <p>Release resources allocated for this tag instance.</p>
     */
    public void release() {

        super.release();
        this.immediate = null;
        this.required = null;
        this.validator = null;

    }


    /**
     * <p>Set properties for the standard attributes common to all
     * components in this library.</p>
     *
     * @param component UIComponent on which to set properties
     */
    protected void setProperties(UIComponent component) {

        super.setProperties(component);
        setBooleanProperty(component, "immediate", this.immediate);
        setBooleanProperty(component, "required", this.required);
        setObjectProperty(component, "validator", this.validator);

    }
    

    // ------------------------------------------------------- Attribute Setters


    public void setImmediate(String immediate) {
        this.immediate = immediate;
    }


    public void setRequired(String required) {
        this.required = required;
    }


    public void setValidator(String validator) {
        this.validator = validator;
    }


}
