/*
 * Copyright 2007 Sun Microsystems, Inc.  All rights reserved.
 *
 * You may not modify, use, reproduce, or distribute this software except in
 * compliance with the terms of the License at:
 * http://developer.sun.com/berkeley_license.html
 */

package com.sun.jmaki.faces.tag;

import javax.faces.component.UIComponent;
import javax.faces.el.ValueBinding;

/**
 * <p>Abstract base class for ValueHolder component tags.</p>
 */
public abstract class AbstractValueHolderComponentTag extends AbstractComponentTag {
    

    // ------------------------------------------------------ Instance Variables


    private String converter;
    private String value;


    // -------------------------------------------------- UIComponentTag Methods


    /**
     * <p>Return the component type required for this tag.</p>
     */
    public abstract String getComponentType();


    /**
     * <p>Return the renderer type required for this tag.</p>
     */
    public abstract String getRendererType();


    /**
     * <p>Release resources allocated for this tag instance.</p>
     */
    public void release() {

        super.release();
        this.converter = null;
        this.value = null;

    }


    /**
     * <p>Set properties for the standard attributes common to all
     * components in this library.</p>
     *
     * @param component UIComponent on which to set properties
     */
    protected void setProperties(UIComponent component) {

        super.setProperties(component);
        setConverterProperty(component, "converter", this.converter);
        setObjectProperty(component, "value", this.value);

    }
    

    // ------------------------------------------------------- Attribute Setters


    public void setConverter(String converter) {
        this.converter = converter;
    }


    public void setValue(String value) {
        this.value = value;
    }


}
