/*
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License
 * (the "License").  You may not use this file except
 * in compliance with the License.
 *
 * You can obtain a copy of the license at
 * glassfish/bootstrap/legal/CDDLv1.0.txt or
 * https://glassfish.dev.java.net/public/CDDLv1.0.html.
 * See the License for the specific language governing
 * permissions and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL
 * HEADER in each file and include the License file at
 * glassfish/bootstrap/legal/CDDLv1.0.txt.  If applicable,
 * add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your
 * own identifying information: Portions Copyright [yyyy]
 * [name of copyright owner]
 */
package com.sun.grizzly.comet;

import com.sun.enterprise.web.connector.grizzly.comet.CometContext;
import com.sun.enterprise.web.connector.grizzly.comet.CometEngine;
import com.sun.enterprise.web.connector.grizzly.comet.CometEvent;
import com.sun.enterprise.web.connector.grizzly.comet.CometHandler;
import java.io.IOException;
import java.io.PrintWriter;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Simple Flickr Magnet Demo.
 *
 * @author Jeanfrancois Arcand
 * @author Greg Murray
 */
public class CometServlet extends HttpServlet{
    
    private static final long DEFAULT_EXPIRATION_DELAY = 60 * 6 * 1000;
    
    /**
     * The Comet enabled context-path
     */
    private String contextPath;
    
    
    /**
     * Moderator Context
     */
    private String moderatorPath;
    
    
    /**
     * Simple counter
     */
    private int counter;
    
    public CometServlet() {
    }
    
    public void init(ServletConfig config) throws ServletException {
        super.init(config);
                
        String cometUrlPattern = config.getInitParameter("cometUrlPattern");
        if (cometUrlPattern == null){
            cometUrlPattern = "/words";
        }
        
        // When the Servlet is initialized, tell Grizzly to dispatch
        // all matching requests to the CometEngine.
        contextPath = config.getServletContext().getContextPath()
            + cometUrlPattern;
        CometEngine cometEngine = CometEngine.getEngine();
        CometContext context = cometEngine.register(contextPath);
        
        moderatorPath = config.getServletContext().getContextPath()
            + "/moderator";        
        CometContext moderatorContext = cometEngine.register(moderatorPath);
        
  	String expire = config.getInitParameter("expirationDelay");
	if (expire == null) {
	    context.setExpirationDelay(DEFAULT_EXPIRATION_DELAY);
	} else {
	    context.setExpirationDelay(Long.parseLong(expire));
	}
        moderatorContext.setExpirationDelay(context.getExpirationDelay());
    }
    
    
    public void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        doPost(request,response);
    }
    
    
    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException {
        
        String action = request.getParameter("action");
        CometEngine cometEngine = CometEngine.getEngine();
        CometContext cometContext = cometEngine.getCometContext(contextPath);
        CometContext moderatorContext = cometEngine.getCometContext(moderatorPath);
        
        if (action != null) {
            if ("post".equals(action)){
                String username = (String) request.getSession(true)
                    .getAttribute("username");
                String message = request.getParameter("message");
                String callback = request.getParameter("callback");
                if (callback== null) {callback = "alert";}
                
                // Notify other registered CometHandler.
                cometContext.notify("<script id='" + username + "_" + counter++ + "'>"
                        + "window.parent." + callback + "(" +  message + ");</script>");
                moderatorContext.notify("User: " + request.getRemoteAddr() +
                        " did (" +  message + ")");                
                response.getWriter().println("ok");
                return;
            } else if ("start".equals(action)) {
                response.setContentType("text/html");
                String username = (String) request.getSession(true)
                .getAttribute("username");
                String callback = request.getParameter("callback");
                if (callback== null) {callback = "alert";}
                
                String message = "{ message : 'Welcome " + username + "'}";
                response.getWriter().println("<script id='" + username + "_" + counter++ + "'>"
                        + "window.parent." + callback + "(" +  message + ");</script>");
                
                // Create and register a CometHandler.
                CometRequestHandler handler = new CometRequestHandler();
                handler.attach(response.getWriter());
                cometContext.addCometHandler(handler);
                moderatorContext.notify("User: " + request.getRemoteAddr() +
                    " has loaded the main page."); 
                return;
            }  else if ("open_moderator".equals(action)) {                
                response.setContentType("text/html");                               
                CometRequestHandler handler = new ModeratorHandler();
                handler.attach(response.getWriter());
                CometEngine.getEngine().getCometContext(moderatorPath)
                        .addCometHandler(handler);

                response.getWriter().println("<h2>Under the hood Window</h2>");
                return;
            }         
        }
    }
    
    // --------------------------------------------------------- Async Hook ---/
    
    public class ModeratorHandler extends CometRequestHandler {
 
       public synchronized void onEvent(CometEvent event) throws IOException{
            try{                
                if (event.getType() != CometEvent.READ ){
                    printWriter.println("<p><pre><code>" + 
                            event.attachment() + "</code></pre></p>");
                    printWriter.flush();
                }
            } catch (Throwable t){
                t.printStackTrace();
            }
        }
    }
    
    
    public class CometRequestHandler implements CometHandler<PrintWriter>{
        
        protected PrintWriter printWriter;
                
        public void attach(PrintWriter printWriter){
            this.printWriter = printWriter;
        }
                
        public synchronized void onEvent(CometEvent event) throws IOException{
            try{                
                if (event.getType() != CometEvent.READ){
                    printWriter.println(event.attachment());
                    printWriter.flush();
                }
            } catch (Throwable t){
                t.printStackTrace();
            }
        }
        
        public void onInitialize(CometEvent event) throws IOException{
            printWriter.println("<!doctype html public \"-//w3c//dtd html 4.0 transitional//en\">");
            printWriter.println("<html><head><title>jMaki Grizzly Comet Words Sample</title></head><body bgcolor=\"#FFFFFF\">");
            printWriter.flush();
        }
        
        
        public void onTerminate(CometEvent event) throws IOException{
            onInterrupt(event);
        }
        
        
        public void onInterrupt(CometEvent event) throws IOException{
            printWriter.println("jMaki Grizzly Comet Words Sample closed<br/>");
            printWriter.println("</body></html>");
            printWriter.flush();
            printWriter.close();
        }
    }
}