/* Copyright 2004 Sun Microsystems, Inc.  All rights reserved.  You may not modify, use, reproduce, or distribute this software except in compliance with the terms of the License at: 
 http://ajax.dev.java.net/LICENSE.txt
 $Id: AjaxContext.java,v 1.5 2007/02/10 04:16:21 gmurray71 Exp $ */

package com.sun.jmaki;

import java.util.Map;
import java.io.Writer;
import java.net.URL;
import javax.servlet.ServletContext;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;

/**
 *  This object contains everything needed to generate the content using the AjaxCommon.
 *
 * @author Greg Murray
 */
public class AjaxContext {
    
    private Writer out = null;
    private ServletRequest request = null;
    private ServletResponse response = null;
    private ServletContext ctx = null;
    private Map properties = null;
    private StringBuffer template = null;
    
    public AjaxContext(ServletContext ctx,
                       ServletRequest req,
                       ServletResponse res,
                       Writer out,
                       Map properties) {
        
        this.properties = properties;
        this.request = req;
        this.response = res;
        this.ctx = ctx;
        this.out = out;
    }
    
    public Writer getWriter() {return out;}
    public ServletRequest getRequest(){return request;}
    public ServletResponse getResponse(){return response;}
    public ServletContext getContext() {return ctx;}
    
    public void setTemplate(StringBuffer template) {
        this.template = template;
    }
    
    public String getString(String key) {
        if (properties.containsKey(key)) {
            return (String)properties.get(key);
        }
        return null;
    }
    
    public boolean getBoolean(String key) {
        if (properties.containsKey(key)) {
            return ((Boolean)properties.get(key)).booleanValue();
        }
        return false;
    }
    
    public Map getProperties() {
        return properties;
    }
    
    public StringBuffer getTemplate() {
        return template;
    }
    
    public String getInitParameter(String key) {
        return ctx.getInitParameter(key);
    }
    
    public Object getAttribute(String key) {
        return ctx.getAttribute(key);
    }
    
    public void setAttribute(String key, Object val) {
        ctx.setAttribute(key, val);
    }
    
    public void setRequestAttribute(String key, Object val) {
        request.setAttribute(key, val);
    }
    
    public Object getRequestAttribute(String key) {
        return request.getAttribute(key);
    }
    
    public URL getResource(String resource) throws java.net.MalformedURLException {
        return ctx.getResource(resource);
    }
    
    public String getContextPath() {
        return ((javax.servlet.http.HttpServletRequest)request).getContextPath();
    }

    public String getScheme() {
        return ((javax.servlet.http.HttpServletRequest)request).getScheme();
    }
    
    public String getRequestURI() {
        return ((javax.servlet.http.HttpServletRequest)request).getRequestURI();
    }
    
    public String getServerName() {
        return request.getServerName();
    }
    
    public int getServerPort() {
        return request.getServerPort();
    }
}

