/* Copyright 2004 Sun Microsystems, Inc.  All rights reserved.  You may not modify, use, reproduce, or distribute this software except in compliance with the terms of the License at: 
 http://bpcatalog.dev.java.net/LICENSE.txt
 $Id: AjaxTag.java,v 1.22 2006/12/15 23:08:59 gmurray71 Exp $ */

package com.sun.jmaki;

import javax.servlet.jsp.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.tagext.*;

import java.io.*;
import java.net.*;
import java.util.*;
import java.util.logging.*;

/**
 * Base tag for AJAX Wrappers.
 *
 * @author Greg Murray
 */
public class AjaxTag extends BodyTagSupport {
 
    private static Logger logger;
    
    private boolean embedded = false;
    
    private HashMap map;
     
    private String id = null;
    private String onLoad = null;
    private String args = null;
    private String name = null;
    private String style = null;
    private String script = null;
    private String template = null;
    private String service = null;
    private String type = null;
    private String value = null;
    private String selected = null;
    private ServletContext ctx = null;
    
    public void setId(String id) {
        getMap().put("uuid", id);
        this.id=id;
    }
    
    public void setName(String name) {
        getMap().put("name", name);
        this.name=name;
    }
    
    public void setArgs(String args) {
        getMap().put("args", args);
        this.args=args;
    }
    
    public void setService(String service) {
        this.service=service;
        getMap().put("service", service);
    }
    
    public void setStyle(String style) {
        getMap().put("style", style);
        this.style=style;
    }
    
    public void setScript(String script) {
        getMap().put("script", script);
        this.script=script;
    }
    
    public void setTemplate(String template) {
        getMap().put("template", template);
        this.template=template;
    }
    
    public void setEmbedded(boolean embedded) {
        getMap().put("embeded", new Boolean(embedded));
        this.embedded=embedded;
    }
    
    public void setSelected(String selected) {
        getMap().put("selected", selected);
        this.selected=selected;
    }
    
    public void setValue(String value) {
        getMap().put("value", value);
        this.value=value;
    }

    public void setOnload(String onload) {
        getMap().put("onload", onload);
        this.onLoad=onLoad;
    }
    
    public int doStartTag() throws JspTagException {
        return EVAL_BODY_BUFFERED;
    }
    
    public int doAfterBody() throws JspTagException {
        return SKIP_BODY; 
    }
    
    private HashMap getMap() {
        if (map == null) {
            map = new HashMap();
        }
        return map;
    }
    
    public int doEndTag() throws JspTagException {
        PageContext pctx = (PageContext)pageContext;
        HttpServletRequest request = (HttpServletRequest)pctx.getRequest();
        HttpServletResponse response = (HttpServletResponse)pctx.getResponse();
        if (ctx == null) ctx = pctx.getServletContext();
        JspWriter out = pageContext.getOut();
        // now that we have everything do the evaluation
        AjaxContext ajx = new AjaxContext(ctx, request, response, out, map);
        AjaxCommon.doProcess(ajx);
        
        reset();
        return EVAL_PAGE;
    }
    
    private void reset() {
        map = null;
	    script = null;
        style = null;
        service = null;
        name = null;
        embedded = false;
        template = null;
        type = null;
        onLoad = null;
        args = null;
        id = null;
    }
    
    public static Logger getLogger() {
        if (logger == null) {
            logger = Logger.getLogger("com.sun.javaee.blueprints.jmaki.Log");
        }
        return logger;
    }

}

