package com.sun.jmaki;

import com.sun.jmaki.AjaxWrapper;

import com.sun.facelets.tag.jsf.ComponentHandler;
import com.sun.facelets.tag.jsf.ComponentConfig;
import com.sun.facelets.FaceletContext;
import com.sun.facelets.tag.TagAttribute;
import javax.faces.event.ValueChangeEvent;
import javax.faces.event.ValueChangeListener;
import javax.faces.el.ValueBinding;
import javax.faces.el.MethodBinding;

/**
 * facelethandler for jMaki's 'ajax' component
 *
 * @author Norbert von Truchsess
 */

public class AjaxWrapperHandler extends ComponentHandler {

	private TagAttribute name;

	private TagAttribute service;

	private TagAttribute selected;

	private TagAttribute template;

	private TagAttribute script;

	private TagAttribute style;

	private TagAttribute args;

	private TagAttribute value;

	private TagAttribute valueChangeListener;
	

	public AjaxWrapperHandler(ComponentConfig config) {
		super(config);
System.out.println("**** ajax wrapper init");          
		name = this.getRequiredAttribute("name");
		service = this.getAttribute("service");
		selected = this.getAttribute("selected");
		template = this.getAttribute("template");
		script = this.getAttribute("script");
		style = this.getAttribute("style");
		args = this.getAttribute("args");
		value = this.getAttribute("value");
		valueChangeListener = this.getAttribute("valueChangeListener");
	}


	protected void setAttributes(FaceletContext ctx, Object instance) {
		super.setAttributes(ctx, instance);
        System.out.println("**** ajax wrapper");  
		AjaxWrapper component = (AjaxWrapper) instance;
		try {
			component.setName(this.name.getValue(ctx));
			if (this.service!=null)	component.setService(this.service.getValue(ctx));
			if (this.style!=null) component.setStyle(this.style.getValue(ctx));
		    if (this.script!=null) component.setScript(this.script.getValue(ctx));
		    if (this.template!=null) component.setTemplate(this.template.getValue(ctx));
		    if (this.selected!=null) component.setSelected(this.selected.getValue(ctx));
		    if (this.args!=null) component.setArgs(this.args.getValue(ctx));
		    if (this.value!=null) {
		    	if (this.value.isLiteral()) {
		    		component.setValue(this.value.getValue(ctx));
		    	} else {
                    ValueBinding vb = ctx.getFacesContext().getApplication().createValueBinding(this.value.getValue(ctx));
                
                    component.setValueBinding("value", vb);
                    // JSF 1.2 way         
		    		//component.setValueExpression("value",this.value.getValueExpression(ctx,String.class));
		    	}
		    }
		    if (this.valueChangeListener!=null) {
		    	Class args[] = { ValueChangeEvent.class };
                MethodBinding vb = ctx.getFacesContext().getApplication().createMethodBinding(this.valueChangeListener.getValue(ctx), args);

                component.setValueChangeListener(vb);
                // JSF 1.2 way             
		    	//component.addValueChangeListener(new ValueChangeListener(this.valueChangeListener.getMethodExpression(ctx, null,args)));
		    }
		} catch (Exception e) {
            throw new javax.faces.FacesException("Unable to process attributes of jMaki component: " + component.getId(),e);
		}
	}
}