/* Copyright 2005 Sun Microsystems, Inc. All rights reserved. You may not modify, use, reproduce, or distribute this software except in compliance with the terms of the License at: http://developer.sun.com/berkeley_license.html
 */
 
/*
 * AjaxWrapperPhaseListener.java
 *
 * Created on May 11, 2005, 11:04 AM
 */

package com.sun.jmaki;

import java.io.*;
import java.net.*;
import java.util.*;
import javax.faces.event.*;
import javax.faces.el.*;
import javax.faces.component.*;
import javax.faces.validator.*;
import javax.servlet.http.*;
import javax.servlet.*;
import javax.faces.context.FacesContext;
import javax.faces.webapp.UIComponentTag;
import java.util.logging.*;

/**
 * Phase listener which responds to requests for the script tag
 * @author Greg Murray
 */
public class AjaxWrapperPhaseListener implements PhaseListener {
 
    public static String JMAKI_AJAX = ".ajax";   
    private static String CACHE = "jmaki_cache";
    private static String CACHE_TIMES = "jmaki_cache_times";
    private static Logger logger;
    public static String cpRoot = "META-INF";
    
    public static Logger getLogger() {
        if (logger == null) {
            logger = Logger.getLogger("com.sun.jmaki.Log");
        }
        return logger;
    }     

    public AjaxWrapperPhaseListener() {}
        
    public void afterPhase(PhaseEvent event) {
	FacesContext context = event.getFacesContext();
	if (context.getResponseComplete()) {
	    // Response has already been handled
	    return;
	}

        String rootId = context.getViewRoot().getViewId();
        // It is not safe to treat a viewId as a resource Id because
        // the viewId may include the FacesServlet mapping.
        if (Util.resourceMapsToFacesServlet(event.getFacesContext(), rootId)) {
            return;
        }
        // Special case.  In the case of prefix mapped requests,
        // the FacesServlet mapping IS NOT included in the viewId.  With 
        // extension mapped requests, it IS included.  Therefore, this 
        // heuristic is required.  Personally, using file extensions
        // to determine behavior is undesirable, but since this class already
        // uses this approach, what damage will one usage more do?
        
        // Heuristic.  If the file is a JSP file, or an xhtml file, take no 
        // action.
        if (rootId.endsWith(".jsp") || rootId.endsWith(".xhtml")) {
            return;
        }
        if (rootId.endsWith(".js") ||
            rootId.endsWith(".htm") ||
            rootId.endsWith(".html")) {
            getResource(event, rootId, true, true, null);
        } else if (rootId.endsWith(".css")) {
              getBinaryResource(event, rootId, true, "text/css");   
        } else if (rootId.endsWith(".gif")) {
              getBinaryResource(event, rootId, true, "image/GIF");     
        } else if (rootId.endsWith(".jpg")) {
              getBinaryResource(event, rootId, true, "image/JPEG");
        } else if (rootId.endsWith(".png")) {
              getBinaryResource(event, rootId, true, "image/png");     
        } else if (rootId.endsWith(JMAKI_AJAX)){
            handleAjaxRequest(rootId, event);
        } else {
              getBinaryResource(event, rootId, true, null);   
	    }
    }
              
    private boolean updateValueBoundObject(FacesContext context, String valueBinding, String value) {
        try {
            ValueBinding vb = context.getApplication().createValueBinding(valueBinding);
            vb.setValue(context, value);
        } catch (EvaluationException ex) {
            return false;
        }
            return true;
    }

    public void beforePhase(PhaseEvent event) {
    }

    public PhaseId getPhaseId() {
        return PhaseId.RESTORE_VIEW;
    }
    
    private void handleAjaxRequest(String rootId, PhaseEvent event) {

        FacesContext context = event.getFacesContext();
        HttpServletResponse response =
            (HttpServletResponse)context.getExternalContext().getResponse();

        Object object = context.getExternalContext().getRequest();

        if (!(object instanceof HttpServletRequest)) {
            // PortletRequest? Handle that here?
            return;
        }

        HttpServletRequest request = (HttpServletRequest)object;      
        
        String cmd = request.getParameter("cmd");
        String uuid = request.getParameter("uuid");
        String argString = request.getParameter("args");
        String[] args = null;
        if (argString != null) {
            args = argString.split(",");
        }
        
        int targetIndex = rootId.lastIndexOf("/");
        String[] nRoot = rootId.split("/");
        // for the case where we are not under an extension mapping
        if (targetIndex < 0) targetIndex = 0;
        String target = rootId.substring(targetIndex, rootId.length());
        String[] bindingArgs = target.split("-");
        String tO =  bindingArgs[0].substring(1,bindingArgs[0].length());
        String tM = bindingArgs[1].substring(0,(bindingArgs[1].length() - ".ajax".length()));
        String bindingExpression = "#{" + tO + "." + tM  + "}";
        try {

            if ((cmd != null) && "update".equals(cmd)) {
                String value = request.getParameter("value");
                if (value != null) {
                    boolean success = updateValueBoundObject(context,bindingExpression, value);
                    response.setHeader("Cache-Control", "no-cache");
                    response.getWriter().write("<status>" + success + "</status>");
                }
            } else {  
                AjaxResult result = invokeBinding(context, bindingExpression);
                if (result.getResponseType() == AjaxResult.XML) {
                    response.setContentType("text/xml");
                }
                response.setHeader("Cache-Control", "no-cache");
                response.getWriter().write(result.toString());
            }
        } catch (EvaluationException ee) {
            getLogger().severe(ee.toString());
        } catch (IOException ioe) {
            getLogger().severe(ioe.toString());
        } finally {
            event.getFacesContext().responseComplete();
            return;
        }
    }

   private AjaxResult invokeBinding(FacesContext context, String methodBinding) {
        if (UIComponentTag.isValueReference(methodBinding)) {
            Class[] argTypes = { FacesContext.class, AjaxResult.class };
            MethodBinding vb = context.getApplication().createMethodBinding(methodBinding, argTypes);
            AjaxResult result = new AjaxResult();
            Object[] args = { context, result };
            vb.invoke(context, args);
            return result;
        } else {
            getLogger().severe("AjaxWrapperPhaseListener Error: Bad method binding: " + methodBinding);
        }
        return null;
    }
    
    public void getBinaryResource(PhaseEvent event, String resource, boolean fromWeb, String contentType) {
        FacesContext context = event.getFacesContext();
        ServletContext ctx =  (ServletContext)context.getExternalContext().getContext();
        InputStream stream = null;
        URLConnection con;
        ByteArrayOutputStream buffer = null;
        try {
            // check the time stamp on the resource
            URL url = ctx.getResource(resource);
            if (url == null) {
                url = AjaxWrapperPhaseListener.class.getClassLoader().getResource(cpRoot + resource);
                if (url == null) {
                    event.getFacesContext().responseComplete();
                    return;
                }
            }
            con = url.openConnection();
            buffer = getBinaryResource(con.getInputStream());
            HttpServletResponse response = (HttpServletResponse)context.getExternalContext().getResponse();
            if (contentType != null) {
                response.setContentType(contentType);
            }
            if (buffer != null) {
                response.getOutputStream().write(buffer.toByteArray());
                buffer.close();
            } 
            event.getFacesContext().responseComplete();
            return;
        } catch (Exception e) {
            getLogger().severe("AjaxWrapperPhaseListener Error: Could not load " + resource);
        }
        event.getFacesContext().responseComplete();
        return;
    }
    
    private ByteArrayOutputStream getBinaryResource(InputStream in) {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        byte[] bytes = new byte[1024];
        try {
          int read = 0;
          while ((read = in.read(bytes)) > 0) {
               out.write(bytes, 0, read);
          }
          in.close();
          out.close();
        } catch (IOException e) {
            getLogger().severe("AjaxWrapperPhaseListener:loadResource from stream error:"  + e);
        }
        return out;
    }
    
   private void getResource(PhaseEvent event, String resource, boolean fromWeb, boolean cacheContent, String contentType) {
        FacesContext context = event.getFacesContext();
        ServletContext ctx =  (ServletContext)context.getExternalContext().getContext();
        HttpServletResponse response = (HttpServletResponse)context.getExternalContext().getResponse();
        InputStream stream = null;
        StringBuffer buffer = null;
        URLConnection con;
        try {
            // try the resource root
            URL url = ctx.getResource(resource);
            // now try the classpath
            if (url == null) {
                url = AjaxWrapperPhaseListener.class.getClassLoader().getResource(cpRoot + resource);
                if (url == null) {
                    event.getFacesContext().responseComplete();
                    return;
                }
            }
            con = url.openConnection();
            if (cacheContent) {
                HashMap cache = (HashMap)ctx.getAttribute(CACHE);
                HashMap cacheTimes = (HashMap)ctx.getAttribute(CACHE_TIMES);
                if (cache == null) {
                    cache = new HashMap();
                    cacheTimes = new HashMap();
                    ctx.setAttribute(CACHE, cache);
                    ctx.setAttribute(CACHE_TIMES, cacheTimes);
                }
                long lastModified = con.getLastModified();
                long cacheModified = 0;
                if (cacheTimes.get(resource) != null) {
                    cacheModified = ((Long)cacheTimes.get(resource)).longValue();
                }
                if (cacheModified < lastModified) {
                    buffer = getResource(con.getInputStream());
                    synchronized(cacheTimes) {
                        cacheTimes.put(resource, new Long(lastModified));
                    }
                    synchronized(cache) {
                        cache.put(resource, buffer);
                    }
                } else {
                    buffer =(StringBuffer)cache.get(resource);
                }
            } else {
                buffer = getResource(con.getInputStream());
            }
            if (buffer != null) response.getWriter().write(buffer.toString());
        } catch (Exception e) {
            getLogger().severe("AjaxWrapperRenderer Error: Could not load " + resource);
        }
        event.getFacesContext().responseComplete();
        return;
    }
    
    private StringBuffer getResource(InputStream stream) {
        StringBuffer buffer = new StringBuffer();
        BufferedReader bufReader = null;
        String curLine = null;
        try {
            bufReader = new BufferedReader(new InputStreamReader(stream));
            while (null != (curLine = bufReader.readLine())) {
                buffer.append(curLine + "\n");
            }
        } catch (Exception e) {
            getLogger().severe("AJAXTag:loadResource from stream error:"  + e);
        }
        return buffer;
    }
    
}
