/* Copyright 2005 Sun Microsystems, Inc. All rights reserved. You may not modify, use, reproduce, or distribute this software except in compliance with the terms of the License at: http://developer.sun.com/berkeley_license.html
 */

/*
 * AjaxWrapperRenderer.java
 *
 * Created on April 29, 2005, 12:30 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */
package com.sun.jmaki;

import java.io.*;
import java.util.*;
import java.net.*;
import javax.faces.component.*;
import javax.faces.el.*;
import javax.faces.context.*;

import javax.servlet.http.*;
import javax.servlet.*;

import javax.faces.render.Renderer;
import javax.faces.application.Application;
import javax.faces.convert.Converter;
import javax.faces.convert.ConverterException;

import java.util.logging.*;


/**
 *
 * This component relies on a cooperating servlet that responds to asynchronous
 * requests.
 *
 * @author Greg Murray
 */
public class AjaxWrapperRenderer extends Renderer {

    private ServletContext ctx;
    private Properties properties;
    private boolean checkForResources = false;
        
    private static String resourcesDir = "resources";
    
    private static Logger logger;
    
    private static int nextId = 0;

    /** Creates a new instance of AjaxDynamicLabelRenderer */
    public AjaxWrapperRenderer() {
    }

    public void encodeBegin(FacesContext context, UIComponent component)
        throws IOException {
    }

    public void encodeChildren(FacesContext context, UIComponent component) {
    }
    
    public void encodeRecursive(FacesContext context, UIComponent component)
        throws IOException {
        encodeBegin(context, component);
        encodeEnd(context,component);
    }

    private void encodeValue(FacesContext context, UIComponent component) throws IOException {
        String value = "";
        if (component instanceof ValueHolder) {
            ValueHolder vh = (ValueHolder) component;
            value = Util.getFormattedValue(context, vh, vh.getValue());
        }
        ResponseWriter writer = context.getResponseWriter();
        writer.write(value);
    }
    
    public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
	// Make sure we should render something...
	if (!component.isRendered()) {
	    return;
	}

        Map headers = context.getExternalContext().getRequestHeaderMap();
        String renderHeader = (String) headers.get("com.sun.faces.avatar.partial");
        
        // If this is an AJAX request, just encode our value
        if (null != renderHeader && renderHeader.equalsIgnoreCase("values")) {
            encodeValue(context,  component);
            return;
        }
        
        AjaxWrapper wrapper = (AjaxWrapper)component;
        
        ctx =  (ServletContext)context.getExternalContext().getContext();
        if (!checkForResources) {
            if (ctx.getInitParameter("jmaki-resources") != null) {
                resourcesDir = ctx.getInitParameter("jmaki-resources");
            }
            checkForResources = true;
        }
        
        Object o1 = context.getExternalContext().getRequest();
        HttpServletRequest request = (HttpServletRequest)o1;
        Object o2 = context.getExternalContext().getResponse();
        HttpServletResponse response = (HttpServletResponse)o2;

        // The way it is now, it simply won't work because it generates a 
        // URI that doesn't cause the FacesServlet to be hit.
        String basePath = request.getRequestURI();

        // find out the root for JSF so we can make requests back
        // this can be set using a servlet context property if it
        // is a pattern based mapping it defaults to '/faces'
        // otherwise it uses the maping  used in the application.
        String jsfRoot = null;
        if (ctx.getInitParameter("jmaki-jsf-root") != null) {
            jsfRoot = ctx.getInitParameter("jmaki-jsf-root");
        } else {
            if (request.getServletPath().indexOf(".") != -1) {
                jsfRoot = "/faces";
            } else {
                jsfRoot = request.getServletPath();
            }
        }
     	String resourceBase = "";
     	// we know the context root is a / so start from postion 1
     	int first = basePath.indexOf("/", 1);
        
     	// This method only supports path based JSF roots
		int last = basePath.lastIndexOf(jsfRoot + "/");
        
		// find the base path from which to start looking for resources from
		// this is needed if we are working in subdirectories
		if (first != -1 && last != -1) {
			resourceBase = basePath.substring(first + 1, last  + 1);
		}
 
        ResponseWriter out = context.getResponseWriter();
        
        // configure the parameters
        HashMap map = new HashMap();
        map.put("jsfRoot", jsfRoot);
        map.put("resourceBase", resourceBase);
        map.put("name", wrapper.getName());
        map.put("uuid", wrapper.getClientId(context));
        map.put("template", wrapper.getTemplate());
        map.put("args", wrapper.getArgs());
                
        String webRoot = "http://" + request.getServerName();
        if (request.getServerPort() != 80) {
                webRoot += ":" + request.getServerPort();
        }
        webRoot += request.getContextPath();
        map.put("webRoot", webRoot);
        
        // if we end with the jMaki extension we need to map the
        // service back into the phaselistener and thus through JSF
        String service = wrapper.getService();
        if (service != null) {
            if (service.endsWith(AjaxWrapperPhaseListener.JMAKI_AJAX)) {
                service = webRoot + jsfRoot + service;
            }
        }
        if (null != service) {
            map.put("service", service);
        }
        ValueBinding vb = component.getValueBinding("value");

        if (vb != null && service == null) {
            String lr = webRoot + jsfRoot;
            if (!lr.endsWith("/")) lr += "/";
            StringTokenizer strTok = new StringTokenizer(vb.getExpressionString(), "#{.}");
            String callback =  lr + strTok.nextToken() + "-" + strTok.nextToken() + ".ajax";
            map.put("service", callback);

        }
        // this block will result in an expected exception if
        // running with JSF 1.1
        try {
            Object value = getCurrentValue(context, wrapper);

            String stringVal = "";
            if (component instanceof ValueHolder) {
                ValueHolder vh = (ValueHolder) component;
                stringVal = Util.getFormattedValue(context,vh, vh.getValue());
            }

            if (value != null) {
                map.put("value", stringVal);
            }

            String mapping = Util.getFacesMapping(context);
            if (null == mapping) {
                throw new IOException("Cannot determine URL mapping for Faces.");
            }
            assert(null != mapping);
            if (Util.isPrefixMapped(mapping)) {
                service = webRoot + "/" + mapping + context.getViewRoot().getViewId();
            }else {
                String viewId = context.getViewRoot().getViewId();
                int lastDot = viewId.lastIndexOf(".");
                service = webRoot + viewId.substring(0, lastDot) + mapping;
            }       

        } catch(RuntimeException pnfx) {
            // exception thrown for JSF 1.1 
            // dynamic faces with a method bound expression
        }
        if (wrapper.getSelected() != null) {
            ValueBinding vb2 = context.getApplication().createValueBinding(wrapper.getSelected());
            if (vb2 != null) {
                String selected = (String)vb2.getValue(context);
                map.put("selected", selected);
            }
        }
        AjaxContext ajx = new AjaxContext(ctx, request, response, out, map);
        AjaxCommon.doProcess(ajx);
	}
      
    public static Logger getLogger() {
        if (logger == null) {
            logger = Logger.getLogger("com.sun.jmaki.Log");
        }
        return logger;
    }
    
    private Renderer textRenderer = null;
    
    public Object getConvertedValue(FacesContext context, UIComponent component,
            Object submittedValue)
            throws ConverterException {
        if (null == textRenderer) {
            textRenderer = Util.getCurrentRenderKit(context).getRenderer("javax.faces.Input",
                    "javax.faces.Text");
        }
        assert(null != textRenderer);
        return textRenderer.getConvertedValue(context, component, submittedValue);
    }

    
    private Object getValue(UIComponent component) {
         if (component instanceof ValueHolder) {
             Object value = ((ValueHolder) component).getValue();
             if (getLogger().isLoggable(Level.FINE)) {
                  getLogger().fine("component.getValue() returned " + value);
             }
             return value;
         }
         return null;
     }
     
     
     private String getCurrentValue(FacesContext context, UIComponent component) {

         if (component instanceof UIInput) {
             Object submittedValue = ((UIInput) component).getSubmittedValue();
             if (submittedValue != null) {
                 return (String) submittedValue;
             }
         }
         String currentValue = null;
         Object currentObj = getValue(component);
         if (currentObj != null) {
             currentValue = getFormattedValue(context, component, currentObj);
         }
         return currentValue;
     }    
     
     private String getFormattedValue(FacesContext context, UIComponent component,
                                        Object currentValue)
         throws ConverterException {

         String result = null;
         // formatting is supported only for components that support
         // converting value attributes.
         if (!(component instanceof ValueHolder)) {
             if (currentValue != null) {
                 result = currentValue.toString();
             }
             return result;
         }

         Converter converter = null;

         // If there is a converter attribute, use it to to ask application
         // instance for a converter with this identifer.
         converter = ((ValueHolder) component).getConverter();

         // if value is null and no converter attribute is specified, then
         // return a zero length String.
         if (converter == null && currentValue == null) {
             return "";
         }

         if (converter == null) {
             // Do not look for "by-type" converters for Strings
             if (currentValue instanceof String) {
                 return (String) currentValue;
             }

             // if converter attribute set, try to acquire a converter
             // using its class type.

             Class converterType = currentValue.getClass();
             converter = getConverterForClass(converterType, context);

             // if there is no default converter available for this identifier,
             // assume the model type to be String.
             if (converter == null) {
                 result = currentValue.toString();
                 return result;
             }
         }

         return converter.getAsString(context, component, currentValue);

     }
     
     private static Converter getConverterForClass(Class converterClass,
                                                  FacesContext context) {
         if (converterClass == null) {
             return null;
         }
         try {            
             Application application = context.getApplication();
             return (application.createConverter(converterClass));
         } catch (Exception e) {
             return (null);
         }
     }

     public void decode(FacesContext context, UIComponent comp) {

         String clientId = comp.getClientId(context);
         AjaxWrapper component = (AjaxWrapper) comp;
         assert (clientId != null);
         Map requestMap = context.getExternalContext().getRequestParameterMap();
         // Don't overwrite the value unless you have to!
         if (requestMap.containsKey(clientId)) {
             String newValue = (String)requestMap.get(clientId);
             component.setSubmittedValue(newValue);
         }
         
     }
}
