/* Copyright 2004 Sun Microsystems, Inc.  All rights reserved.  You may not modify, use, reproduce, or distribute this software except in compliance with the terms of the License at: 
 http://bpcatalog.dev.java.net/LICENSE.txt
 $Id: AjaxWrapperTag.java,v 1.9 2006/12/15 23:08:59 gmurray71 Exp $ */

package com.sun.jmaki;

import javax.servlet.jsp.*;
import javax.servlet.*;
import javax.servlet.jsp.tagext.*;

import java.io.*;
import java.net.*;
import java.util.*;

import javax.faces.component.*;
import javax.faces.context.FacesContext;
import javax.faces.el.*;
import javax.faces.event.ValueChangeEvent;
import javax.faces.webapp.UIComponentBodyTag;

/**
 * Base tag for AJAX Wrappers.
 *
 * @author Greg Murray
 */
public class AjaxWrapperTag extends UIComponentBodyTag {
        
    private boolean fromWeb = true;
    private boolean embedded = false;
    private String name = null;
    private String style = null;
    private String script = null;
    private String template = null;
    private String service = null;
    private String selected = null;
    private String args = null;
    private String onload = null;
    private String value = null;
    private String valueChangeListener = null;
    private ServletContext ctx = null;
    
    public void setName(String name) {this.name=name;}
    public void setService(String service) {this.service=service;}
    public void setStyle(String style) {this.style=style;}
    public void setScript(String script) {this.script=script;}
    public void setTemplate(String template) {this.template=template;}
    public void setSelected(String selected) {this.selected=selected;}
    public void setArgs(String args) {this.args=args;}
    public void setValue(String value) {this.value=value;}
    public void setValueChangeListener(java.lang.String valueChangeListener) {
        this.valueChangeListener = valueChangeListener;
    }
    
    /**
        * Gets the type of the component associated wit the tag.
     * @return the name of the component type
     */
    public String getComponentType() {
        return "AjaxWrapper";
    }
    
    /**
        * Gets the type of the renderer that will render this tag.
     * @return the type of the renderer that renders this tag
     */
    public String getRendererType() {
        return "AjaxWrapperType";
    }
    
    /**
        * Releases resources allocated during the execution of this tag handler.
     */
    public void release() {
        super.release();
    }
      
    /**
        * Sets the properties of the specified component.
     * @param component the component associated with this tag
     */
    protected void setProperties(UIComponent component) {
        AjaxWrapper wrapper = null;     
        try {
            wrapper = (AjaxWrapper)component;
        } catch (ClassCastException cce) {
            throw new IllegalStateException("Component " + component.toString() +
                                            " not expected type.  Expected: AjaxWrapper. Make sure you defined the tag under the view root.");
        }
        
        wrapper.setName(name);
        wrapper.setService(service);
        wrapper.setStyle(style);
        wrapper.setScript(script);
        wrapper.setTemplate(template);
        wrapper.setSelected(selected);
        wrapper.setArgs(args);
        
        if (value != null) {
            if (isValueReference(value)) {
                ValueBinding vb = getFacesContext().getApplication().createValueBinding(value);
                wrapper.setValueBinding("value", vb);
            } else {
                wrapper.setValue(value);
            }
        } 
        
        if (valueChangeListener != null) {
            if (isValueReference(valueChangeListener)) {
                Class args[] = { ValueChangeEvent.class };
                MethodBinding vb = FacesContext.getCurrentInstance().getApplication().createMethodBinding(valueChangeListener, args);
                wrapper.setValueChangeListener(vb);
            } else {
                throw new javax.faces.FacesException("Unable to find the value for the jMaki component with id: " + 
                                                     this.getId());
            }
        }
    }
    
    public int doAfterBody() throws javax.servlet.jsp.JspException {
        AjaxWrapper mwrapper = null;
        try {
            mwrapper = (AjaxWrapper)getComponentInstance();
        } catch (ClassCastException cce) {
            throw new IllegalStateException("Component type wrong.  Expected: AjaxWrapper.");
        } catch (NullPointerException npx) {
            throw new IllegalStateException("Component not found.  Expected: AjaxWrapper.");
        }
        return super.doAfterBody();
    }
}