/* Copyright 2006 Sun Microsystems, Inc.  All rights reserved.  You may not modify, use, reproduce, or distribute this software except in compliance with the terms of the License at: 
 http://developer.sun.com/berkeley_license.html
 $Id: JmakiController.java,v 1.3 2007/04/11 17:48:26 ntruchsess Exp $ 
*/

package com.sun.jmaki;


import java.io.*;
import java.util.*;
import java.util.logging.*;

import javax.servlet.*;
import javax.servlet.http.*;

public class JmakiController extends HttpServlet {
	
    private static boolean rDebug = false;
    private Logger logger = null;
    private ServletContext context;

    
    /** The Jmaki Controller Serlvet is responsible for loading resources for jMaki
     * which may be services out of a jar file or JavaScript libraries.
     *
     * @author Greg Murray
     */
    public JmakiController() {
    }
    
    public void init(ServletConfig config) throws ServletException {
	    super.init(config);
		context = config.getServletContext();
	}
    
    public Logger getLogger() {
        if (logger == null) {
            logger = Logger.getLogger("com.sun.jmaki.Log");
        }
        return logger;
    }
    private void logMessage(String message) {
        if (rDebug) {
            getLogger().info(message);
        }
    }
   
    public void doGet(HttpServletRequest req, HttpServletResponse res) {
        String target = null;
        try {
            if (req.getParameter("url") != null) {
                target = req.getParameter("url");
                if (!target.startsWith("/")) target = "/" + target;
            } else {
                target = req.getPathInfo();
            }
            InputStream in = JmakiController.class.getClassLoader().getResourceAsStream("META-INF" + target);
            if (in != null) {
                if (target.endsWith(".xml")) {
                    res.setContentType("text/xml");
                } else if (target.endsWith(".js")) {
                    res.setContentType("text/javascript");
                }
                OutputStream out = res.getOutputStream();
                writeBinaryResource(in,out);
            } else {
                PrintWriter out = res.getWriter();
                out.println("Unable to locate resource: " + target);
            }
		} catch (java.io.IOException iox) {
			getLogger().severe("ControllerServlet: caught " + iox);
		}
	}
    
    private void writeBinaryResource(InputStream in, OutputStream out) {
        byte[] bytes = new byte[1024];
        try {
          int read = 0;
          while ((read = in.read(bytes)) > 0) {
               out.write(bytes, 0, read);
          }
          in.close();
          out.close();
        } catch (IOException e) {
            System.out.println("ControllerServlet:loadResource from stream error:"  + e);
        }
    }
    
}
