package jmaki.runtime;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Iterator;
import java.util.Map;

import jmaki.runtime.config.WidgetConfigException;

import org.json.JSONObject;
import org.json.JSONException;

/**
 * @author edburns, Norbert von Truchsess
 */
public class IOUtil {

	public static JSONObject loadJSONObject(InputStream in) throws JSONException, IOException {
		ByteArrayOutputStream out = null;
		byte[] buffer = new byte[1024];
		int read = 0;
		out = new ByteArrayOutputStream();
		try {
			while (true) {
				read = in.read(buffer);
				if (read <= 0)
					break;
				out.write(buffer, 0, read);
			}
			out.flush();
			return new JSONObject(out.toString());
		} finally {
			out.close();
			in.close();
		}
	}

	public static void replace(StringBuffer buff, String target, String replacement) {
		if (buff == null || target == null || replacement == null) {
			return;
		}
		int index = 0;
		while (index < buff.length()) {
			index = buff.indexOf(target);
			if (index == -1) {
				break;
			}
			buff.replace(index, index +  target.length(), replacement);
			index += replacement.length() + 1;
		}
	}

	/**
	 * loads a Resource from an InputStream into a StringBuffer.
	 * @param stream
	 * @param encoding
	 * @return StringBuffer containing all characters read from InputStream
	 * @throws IOException 
	 */
	public static StringBuffer loadResource(InputStream stream, String encoding) throws IOException{
		StringBuffer buffer = new StringBuffer();
		BufferedReader bufReader = null;
		String curLine = null;
		try {
			final InputStreamReader inputStreamReader = (  encoding == null ) ?new InputStreamReader(stream) : new InputStreamReader(stream,
					encoding);
			bufReader = new BufferedReader(inputStreamReader);
			while (null != (curLine = bufReader.readLine())) {
				buffer.append(curLine + "\n");
			}
		} finally {
			if (bufReader != null) {
				bufReader.close();
			}
		}
		return buffer;
	}

	/**
	 *   Convert a Map to a JavaScript object literal StringBuffer
	 *
	 */
	public static StringBuffer createJavaScriptObject(Map p) {
		return createJavaScriptObject(p, false);
	}

	/**
	 *  Convert a Map to a JavaScript object literal or JSON StringBuffer
	 *  Values that are Maps are considered sub-objects and this method is
	 *  recursively called on those objects.
	 *
	 */
	public static StringBuffer createJavaScriptObject(Map p, boolean json) {
		StringBuffer buff = new StringBuffer("{");
		Iterator it = p.entrySet().iterator();

		while (it.hasNext()) {
			String key = (String)((Map.Entry)it.next()).getKey();
			Object val = p.get(key);
			// a value is a map it is an embedded object so call this method recursively
			if ((val != null) && val instanceof java.util.Map) {
				val = createJavaScriptObject((Map)val);
			}
			// check if the value is a number
			if (json) {
				buff.append("'" + key + "':");
			} else {
				buff.append(key + ":");
			}

			if (val == null) {
				buff.append("null");
			} else if (val instanceof Float ||
					val instanceof Long ||
					val instanceof Integer ||
					val instanceof StringBuffer) {
				buff.append(val.toString());
				// if the value is an object literal or an array as a String
			} else if (val instanceof String &&
					((String)val).trim().startsWith("{") ||
					((String)val).trim().startsWith("[")
			) {
				buff.append(val.toString());
			} else {
				buff.append("'" + val.toString()  + "'");
			}
			if (it.hasNext()) {
				buff.append(",");
			}
		}
		buff.append("}");
		return buff;
	}

	public static String resolvePath(String rootPath,String relativePath) throws WidgetConfigException {
		if (rootPath == null || relativePath.startsWith("/") || isExternalUri(relativePath))
			return relativePath;
		int relativePathLen=relativePath.length();
		int relativePos = 0;
		for (;relativePos<relativePathLen && relativePath.indexOf("../",relativePos)==relativePos;relativePos+=3);
		int numberOfDirUp = relativePos>0 ? relativePos / 3 : 0;
		boolean rootPathEndsWithSlash = rootPath.endsWith("/");
		if (numberOfDirUp>0) {
			int rootPos = rootPathEndsWithSlash ? rootPath.length()-1 : rootPath.length();
			int positionOfSlash;
			while ((positionOfSlash=rootPath.lastIndexOf('/',rootPos))>=0 && numberOfDirUp>0) {
				rootPos=positionOfSlash;
				numberOfDirUp--;
			}
			if (numberOfDirUp>0) throw new WidgetConfigException("unable to combine '"+rootPath+"' with relative path '"+relativePath+"'");
			return rootPath.substring(0,rootPos)+'/'+relativePath.substring(relativePos);
		}
		return rootPathEndsWithSlash ? rootPath+relativePath : rootPath+'/'+relativePath;
	}
	
	public static boolean isExternalUri(String uri) {
		return uri.startsWith("http://") || uri.startsWith("https://");
	}
}
