package jmaki.runtime;

import javax.servlet.ServletContext;
import java.net.URL;
import java.net.MalformedURLException;
import java.util.HashMap;

/**
 * @author Norbert von Truchsess
 *
 */
public class ResourceManager {
	
	private ServletContext context;
	private HashMap webResources;
	private HashMap classPathResources;
	private boolean usecache;
	private String resoucesDir;
	
	private static final String CLASSPATHPREFIX="/META-INF";
	
	/**
	 * @param ctx
	 * @param usecache
	 */
	public ResourceManager(ServletContext ctx, boolean usecache) {
		this.context = ctx;
		this.usecache = usecache;
		if (usecache) {
			this.webResources = new HashMap();
			this.classPathResources = new HashMap();
		}
		if ((this.resoucesDir = ctx.getInitParameter(AjaxInitParameters.RESOURCESDIR))==null)
			this.resoucesDir = "/resources";
	}

	/**
	 * @param resource
	 * @return
	 * @throws WidgetResourceException
	 */
	public boolean isWebAppResource(String resource) throws WidgetResourceException {       
		return locateWebAppResource(resource)!=null;
	}
        
	/**
	 * @param resource
	 * @return
	 * @throws WidgetResourceException
	 */
	public URL locateWebAppResource(String resource) throws WidgetResourceException {
		if (usecache && webResources.containsKey(resource)) {                 
			return (URL)webResources.get(resource);
		} else {
			try {
				URL url = context.getResource(resource);                           
				if (usecache)
					webResources.put(resource,url);
				return url;
			} catch (MalformedURLException mue) {
				throw new WidgetResourceException("error locating resource: '"+resource+"'",mue);
			}
		}
	}
        
	/**
	 * @param resource
	 * @return
	 */
	public boolean isClassPathResource(String resource) {
		return locateClassPathResource(resource)!=null;
	}
	
	/**
	 * @param resource
	 * @return
	 */
	public URL locateClassPathResource(String resource) {
		if (usecache && classPathResources.containsKey(resource)) {
			return (URL)classPathResources.get(resource);
		} else {
			URL url = getClass().getResource(CLASSPATHPREFIX+resource);
			if (usecache)	
				classPathResources.put(resource, url);
			return url;
		}
	}

	
	
//	/**
//	 * @param resource
//	 * @param ajx
//	 * @param uriManager
//	 * @param isWebAppResource
//	 * @return
//	 */
//        //gmurray71 : TODO The assumption can not be made here that all resources are located under the resources directory
//        
//	public String buildResourceUri(String resource, AjaxContext ajx,
//			UriManager uriManager, boolean isWebAppResource) throws AjaxException {
//		return isWebAppResource ? 
//                   
//				uriManager.buildWebAppResourceReference(ajx,getWebResourcePath() + resource)
//				// TODO issue with JSP reference-implementation: use of webResourcePath instead of classPathResourcepath is somewhat a hack - access to resources prefixed by 'META-INF' is blocked by faces-servlet even before calling any phaselisteners 
//				: uriManager.buildClassPathResourceReference(ajx, getWebResourcePath()+ resource);
//	}
	
	public String getClassPathResourcePath() {
		// TODO implement getClassPathResourcePath();
		return CLASSPATHPREFIX+getResourcesDir();
	}
	
	public String getWebResourcePath() {
		// TODO implement getWebResourcePath();
		return getResourcesDir();
	}
	
    public String getResourcesDir() {
    	return this.resoucesDir;
    }
    
    public String getWidgetDir(String widgettype) {
    	return "/"+widgettype.replace('.','/');
    }
}
