package jmaki.runtime.config;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.NoSuchElementException;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import jmaki.runtime.ResourceManager;
import jmaki.runtime.AjaxGlobalObjects;
import jmaki.runtime.AjaxException;
import jmaki.runtime.IOUtil;

public class GlobalConfig {
    
    private static final String GLOBALCONFIG="/config.json";
    private static final String WIDGETCONFIG="/widget.json";
    
    private HashMap globalTypes = null; // maybe null if all config-elements are in widget.json
    
    private ApiKeyMap globalApiKeys = null; // maybe null if all config-elements are in widget.json
    
    private HashMap widgetConfigs = new HashMap(); // without widgetConfigs there ain't no widgets
    
    private ResourceManager resourceManager;
    
    public static final Iterator NULLITERATOR = (Iterator) new Iterator() {
        public Object next() throws NoSuchElementException { throw new NoSuchElementException(); }
        public boolean hasNext() { return false; }
        public void remove() throws IllegalStateException { throw new IllegalStateException("next has not been called yet"); }
    };
    
    private boolean isGlobalConfigWebAppResource = false;
    
    public boolean isGlobalConfigWebAppResource() {	return this.isGlobalConfigWebAppResource; }
    
    public GlobalConfig(ResourceManager resourceManager) throws AjaxException {
        this.resourceManager = resourceManager;
        if (resourceManager.isClassPathResource(GLOBALCONFIG))
            loadGlobalConfig(resourceManager.locateClassPathResource(GLOBALCONFIG));
        if (resourceManager.isWebAppResource(GLOBALCONFIG)) {
            loadGlobalConfig(resourceManager.locateWebAppResource(GLOBALCONFIG));
            isGlobalConfigWebAppResource = true;
        }
    }
    
    public Config getConfig(String widgetname, String location) throws AjaxException {
        WidgetConfig widgetConfig = null;
        Type type = null;
        if (location != null && (widgetConfig = (WidgetConfig)widgetConfigs.get(location))==null) {
            widgetConfig = loadWidgetConfig(widgetname, location);
            widgetConfigs.put(location,widgetConfig);
        }
        int positionOfDot;
        if ((type=widgetConfig.getType())==null && globalTypes!=null) {
        	if ((type=(Type)globalTypes.get(widgetname))==null && ((positionOfDot=widgetname.indexOf('.'))<1))
        		type = (Type)globalTypes.get(widgetname.substring(0,positionOfDot));
        }
        return new ConfigImpl(type,globalApiKeys,widgetConfig.getApiKeyMap());
    }
    
    public void discard() {
        if (this.globalApiKeys!=null) {
            this.globalApiKeys.discard();
            this.globalApiKeys=null;
        }
        if (this.globalTypes!=null) {
            Iterator globalTypesIterator = this.globalTypes.entrySet().iterator();
            while (globalTypesIterator.hasNext()) {
                ((Type)((Map.Entry)globalTypesIterator.next()).getValue()).discard();
            }
            this.globalTypes.clear();
            this.globalTypes=null;
        }
        if (this.widgetConfigs!=null) {
            Iterator widgetConfigsIterator = widgetConfigs.entrySet().iterator();
            while (widgetConfigsIterator.hasNext()) {
                ((WidgetConfig)((Map.Entry)widgetConfigsIterator.next()).getValue()).discard();
            }
            this.widgetConfigs.clear();
            this.widgetConfigs=null;
        }
    }
    
    private synchronized void loadGlobalConfig(URL url) throws AjaxException {
        InputStream is = null;
        try {
            is = url.openConnection().getInputStream();
            if (is != null) {
                JSONObject jAll = IOUtil.loadJSONObject(is);
                if (jAll.has("config")) {
                    JSONObject dConfig = jAll.getJSONObject("config");
                    if (dConfig.has("types")) {
                        JSONArray typesArray = dConfig.getJSONArray("types");
                        for (int l = 0; l < typesArray.length(); l++) {
                            JSONObject jType = typesArray.getJSONObject(l);
                            String keytype = jType.getString("id");
                            if (globalTypes==null)
                                globalTypes=new HashMap();
                            globalTypes.put(keytype,loadType(null,jType));
                        }
                    }
                    if (dConfig.has("apikeys")) {
                        JSONArray apikeyArray = dConfig.getJSONArray("apikeys");
                        for (int l = 0; l < apikeyArray.length(); l++) {
                            JSONObject jApiKeys = apikeyArray.getJSONObject(l);
                            String keytype = jApiKeys.getString("id");
                            if (globalApiKeys==null)
                                globalApiKeys=new ApiKeyMap();
                            List keyList = loadApiKeys(jApiKeys,(List)globalApiKeys.get(keytype));
                            if (keyList != null && !globalApiKeys.containsKey(keytype)) {
                                globalApiKeys.put(keytype, keyList);
                            }
                        }
                    }
                }
            }
        } catch (JSONException ex) {
            throw new WidgetConfigException("error parsing 'config.json' from location: '"+url+"': "+ex.getMessage(),ex);
        } catch (IOException ioe) {
            throw new WidgetConfigException("error loading 'config.json' from location: '"+url+"': "+ioe.getMessage(),ioe);
        } finally {
            if (is!=null) {
                try {
                    is.close();
                } catch (IOException ioe) {
                    throw new WidgetConfigException("error closing inputstream for url: '"+url+"'",ioe);
                }
            }
        }
    }

    private synchronized WidgetConfig loadWidgetConfig(String widgetname, String location) throws AjaxException {
        URL url = null;
        WidgetConfig widgetConfig = new WidgetConfig();
        String widgetPath = location+WIDGETCONFIG;
        if (resourceManager.isWebAppResource(widgetPath)) {
        	url = resourceManager.locateWebAppResource(widgetPath);
        } else
        	if (resourceManager.isClassPathResource(widgetPath))
        		url = resourceManager.locateClassPathResource(widgetPath);
        InputStream is = null;
        try {
            if (url != null) is = url.openConnection().getInputStream();
            if (is != null) {
                JSONObject jAll = IOUtil.loadJSONObject(is);
                if (jAll.has("config")) {
                    JSONObject dConfig = jAll.getJSONObject("config");
                    if (dConfig.has("type")) {
                        JSONObject jType = dConfig.getJSONObject("type");
                        int positionOfDot = widgetname.indexOf('.');
                        String widgettype = positionOfDot>0 ? widgetname.substring(0,positionOfDot) : widgetname;
                        String keytype = jType.getString("id");
                        if (keytype.equals(widgetname) || keytype.equals(widgettype))
                            widgetConfig.setType(loadType(location,jType));
                        else throw new WidgetConfigException("unexpected element 'id="+keytype+"' in widget.json for '"+widgetname+"'");
                    }
                    if (dConfig.has("apikeys")) {
                        JSONArray apikeyArray = dConfig.getJSONArray("apikeys");
                        if (apikeyArray.length()>0) {
                        	ApiKeyMap apiKeys = new ApiKeyMap();
                        	for (int l = 0; l < apikeyArray.length(); l++) {
                        		JSONObject jApiKeys = apikeyArray.getJSONObject(l);
                        		String keytype = jApiKeys.getString("id");
                        		List keyList = loadApiKeys(jApiKeys,(List)apiKeys.get(keytype));
                        		if (keyList != null && !apiKeys.containsKey(keytype)) {
                        			apiKeys.put(keytype, keyList);
                        		}
                        	}
                        	widgetConfig.setApiKeys(apiKeys);
                        }
                    }
                }
            }
        } catch (JSONException ex) {
            throw new WidgetConfigException("error parsing 'widget.json' from location: '"+url+"': "+ex.getMessage(),ex);
        } catch (IOException ioe) {
            throw new WidgetConfigException("error loading 'widget.json' from location: '"+url+"': "+ioe.getMessage(),ioe);
        } finally {
            if (is!=null) {
                try {
                    is.close();
                } catch (IOException ioe) {
                    throw new WidgetConfigException("error closing inputstream for url: '"+url+"'",ioe);
                }
            }
        }
        return widgetConfig;
    }
    
    private Type loadType(String currentDir, JSONObject jType) throws JSONException, WidgetConfigException {
        Type type = new Type();
        if (jType.has("styles")) {
            JSONArray nA = jType.getJSONArray("styles");
            for (int il = 0; il < nA.length(); il++) {
                String s = nA.getString(il);
                if (type.getStyles()==null)
                    type.setStyles(new ArrayList());
                type.getStyles().add(IOUtil.resolvePath(currentDir,s));
            }
        }
        if (jType.has("libs")) {
            JSONArray nA = jType.getJSONArray("libs");
            for (int il = 0; il < nA.length(); il++) {
                String s = nA.getString(il);
                if (type.getLibs()==null)
                    type.setLibs(new ArrayList());
                type.getLibs().add(IOUtil.resolvePath(currentDir,s));
            }
        }
        if (jType.has("preload"))
            type.setPreLoad(jType.getString("preload"));
        if (jType.has("postload"))
            type.setPostLoad (jType.getString("postload"));
        if (jType.has("apikey"))
            type.setApiKeyType(jType.getString("apikey"));
        return type;
    }
    
    private List loadApiKeys(JSONObject jApiKeys, List keyList) throws JSONException {
        JSONArray keys = jApiKeys.getJSONArray("keys");
        int keyListLen = keys.length();
        if (keyListLen>0) {
            if (keyList==null)
                keyList=new ArrayList();
            for (int k = 0; k < keyListLen; k++) {
                JSONObject apiKey = keys.getJSONObject(k);
                String targeturl = apiKey.getString("url");
                if (targeturl.endsWith("/"))
                    targeturl=targeturl.substring(0, targeturl.length()-1);
                keyList.add(new ApiKey(targeturl,apiKey.getString("key")));
            }
        }
        return keyList;
    }
}
