/* Copyright 2005 Sun Microsystems, Inc. All rights reserved. You may not modify, use, reproduce, or distribute this software except in compliance with the terms of the License at: http://developer.sun.com/berkeley_license.html
 */

/*
 * AjaxWrapperRenderer.java
 *
 * Created on April 29, 2005, 12:30 PM
 *
 * To change this template, choose Tools | Options and locate the template under
 * the Source Creation and Management node. Right-click the template and choose
 * Open. You can then make changes to the template in the Source Editor.
 */
package jmaki.runtime.jsf;

import java.io.*;
import java.util.*;
import javax.faces.component.*;
import javax.faces.el.*;
import javax.faces.context.*;

import javax.servlet.http.*;
import javax.servlet.*;

import javax.faces.render.Renderer;
import javax.faces.application.Application;
import javax.faces.convert.Converter;
import javax.faces.convert.ConverterException;

import jmaki.runtime.AjaxCommonRenderer;
import jmaki.runtime.AjaxInitParameters;
import jmaki.runtime.AjaxContext;
import jmaki.runtime.UriManager;

import java.util.logging.*;


/**
 *
 * This component relies on a cooperating servlet that responds to asynchronous
 * requests.
 *
 * @author Greg Murray
 */
public class AjaxWrapperRenderer extends Renderer {

    private ServletContext ctx;
//    private Properties properties;
//    private boolean checkForResources = false;
//        
//    private static String resourcesDir = "resources";
    
    private static Logger logger;
    
   private static int nextId = 0;
//
    /** Creates a new instance of AjaxDynamicLabelRenderer */
    public AjaxWrapperRenderer() {
    }

    public void encodeBegin(FacesContext context, UIComponent component)
        throws IOException {
    }

    public void encodeChildren(FacesContext context, UIComponent component) {
    }
    
    public void encodeRecursive(FacesContext context, UIComponent component)
        throws IOException {
        encodeBegin(context, component);
        encodeEnd(context,component);
    }

    private void encodeValue(FacesContext context, UIComponent component) throws IOException {
        String value = "";
        if (component instanceof ValueHolder) {
            ValueHolder vh = (ValueHolder) component;
            value = Util.getFormattedValue(context, vh, vh.getValue());
        }
        ResponseWriter writer = context.getResponseWriter();
        writer.write(value);
    }
    
    public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
    	Map headers = context.getExternalContext().getRequestHeaderMap();
        //gmuray71 : Rest the uuid after 50000
        if (nextId > 50000) {
            nextId = 0;
        } 
    	String renderHeader = (String) headers.get("com.sun.faces.avatar.partial");

    	// If this is an AJAX request, just encode our value
    	if (null != renderHeader && renderHeader.equalsIgnoreCase("values")) {
    		encodeValue(context,  component);
    		return;
    	}

    	AjaxWrapper wrapper = (AjaxWrapper)component;

    	ctx =  (ServletContext)context.getExternalContext().getContext();

    	Object o1 = context.getExternalContext().getRequest();
    	HttpServletRequest request = (HttpServletRequest)o1;
    	Object o2 = context.getExternalContext().getResponse();
    	HttpServletResponse response = (HttpServletResponse)o2;
    	ResponseWriter out = context.getResponseWriter();

    	AjaxContext ajx = AjaxContext.getInstance(ctx, request, response, out);
    	AjaxCommonRenderer ajaxCommon = new AjaxCommonRenderer(ajx);

    	ajaxCommon.setName(wrapper.getName());
        // gmurray71 
        // unique ids are unique to a page and not to an session meaning that if we inject
        // content from one page into another the same widget can be given the same id thus
        // the same id appears 2x in the same page.
        ajaxCommon.setUuid(wrapper.getName().replace('.', '_') + "_" + nextId++);
    	//ajaxCommon.setUuid(wrapper.getClientId(context));
    	ajaxCommon.setTemplate(wrapper.getTemplate());
    	ajaxCommon.setArgs(wrapper.getArgs());

    	UriManager uriManager = ajx.getUriManager();
            
    	ValueBinding vb = component.getValueBinding("service");
    	String service = null;
    	if (vb!=null) {
    		service = uriManager.buildDynamicResourceReference(ajx,vb.getExpressionString());
    	} else {
    		service = wrapper.getService();
    		if (service != null) {
    			int locationOfAjax;
    			if (!service.startsWith("http") && ((locationOfAjax=service.lastIndexOf(AjaxWrapperPhaseListener.JMAKI_AJAX))>0)) {
    				service = uriManager.buildDynamicResourceReference(ajx,service.substring(0,locationOfAjax));
    			}
    		}
    	}

    	if (service == null) {
    		vb = component.getValueBinding("value");
    		if (vb != null) {
    			service = uriManager.buildDynamicResourceReference(ajx,vb.getExpressionString());
    		}
    	}
    	ajaxCommon.setService(service);

    	// this block will result in an expected exception if
    	// running with JSF 1.1
    	try {
    		Object value = getCurrentValue(context, wrapper);

    		String stringVal = "";
    		if (component instanceof ValueHolder) {
    			ValueHolder vh = (ValueHolder) component;
    			stringVal = Util.getFormattedValue(context,vh, vh.getValue());
    		}

    		if (value != null) {
    			ajaxCommon.setValue(stringVal);
    		}
    	} catch(RuntimeException pnfx) {
    		// exception thrown for JSF 1.1 
    		// dynamic faces with a method bound expression
    	}
    	if (wrapper.getSelected() != null) {
    		ValueBinding vb2 = context.getApplication().createValueBinding(wrapper.getSelected());
    		if (vb2 != null) {
    			String selected = (String)vb2.getValue(context);
    			ajaxCommon.setSelected(selected);
    		}
    	}
    	ajaxCommon.doProcess();
    }
      
    public static Logger getLogger() {
        if (logger == null) {
            logger = Logger.getLogger("com.sun.jmaki.Log");
        }
        return logger;
    }
    
    private Renderer textRenderer = null;
    
    public Object getConvertedValue(FacesContext context, UIComponent component,
            Object submittedValue)
            throws ConverterException {
        if (null == textRenderer) {
            textRenderer = Util.getCurrentRenderKit(context).getRenderer("javax.faces.Input",
                    "javax.faces.Text");
        }
        assert(null != textRenderer);
        return textRenderer.getConvertedValue(context, component, submittedValue);
    }

    
    private Object getValue(UIComponent component) {
         if (component instanceof ValueHolder) {
             Object value = ((ValueHolder) component).getValue();
             if (getLogger().isLoggable(Level.FINE)) {
                  getLogger().fine("component.getValue() returned " + value);
             }
             return value;
         }
         return null;
     }
     
     
     private String getCurrentValue(FacesContext context, UIComponent component) {

         if (component instanceof UIInput) {
             Object submittedValue = ((UIInput) component).getSubmittedValue();
             if (submittedValue != null) {
                 return (String) submittedValue;
             }
         }
         String currentValue = null;
         Object currentObj = getValue(component);
         if (currentObj != null) {
             currentValue = getFormattedValue(context, component, currentObj);
         }
         return currentValue;
     }    
     
     private String getFormattedValue(FacesContext context, UIComponent component,
                                        Object currentValue)
         throws ConverterException {

         String result = null;
         // formatting is supported only for components that support
         // converting value attributes.
         if (!(component instanceof ValueHolder)) {
             if (currentValue != null) {
                 result = currentValue.toString();
             }
             return result;
         }

         Converter converter = null;

         // If there is a converter attribute, use it to to ask application
         // instance for a converter with this identifer.
         converter = ((ValueHolder) component).getConverter();

         // if value is null and no converter attribute is specified, then
         // return a zero length String.
         if (converter == null && currentValue == null) {
             return "";
         }

         if (converter == null) {
             // Do not look for "by-type" converters for Strings
             if (currentValue instanceof String) {
                 return (String) currentValue;
             }

             // if converter attribute set, try to acquire a converter
             // using its class type.

             Class converterType = currentValue.getClass();
             converter = getConverterForClass(converterType, context);

             // if there is no default converter available for this identifier,
             // assume the model type to be String.
             if (converter == null) {
                 result = currentValue.toString();
                 return result;
             }
         }

         return converter.getAsString(context, component, currentValue);

     }
     
     private static Converter getConverterForClass(Class converterClass,
                                                  FacesContext context) {
         if (converterClass == null) {
             return null;
         }
         try {            
             Application application = context.getApplication();
             return (application.createConverter(converterClass));
         } catch (Exception e) {
             return (null);
         }
     }

     public void decode(FacesContext context, UIComponent comp) {

         String clientId = comp.getClientId(context);
         AjaxWrapper component = (AjaxWrapper) comp;
         assert (clientId != null);
         Map requestMap = context.getExternalContext().getRequestParameterMap();
         // Don't overwrite the value unless you have to!
         if (requestMap.containsKey(clientId)) {
             String newValue = (String)requestMap.get(clientId);
             component.setSubmittedValue(newValue);
         }
         
     }
}