package jmaki.runtime.jsf;

import com.sun.facelets.tag.jsf.ComponentHandler;
import com.sun.facelets.tag.jsf.ComponentConfig;
import com.sun.facelets.FaceletContext;
import com.sun.facelets.tag.TagAttribute;
import javax.faces.event.ValueChangeEvent;
import javax.faces.event.MethodExpressionValueChangeListener;
import javax.faces.el.ValueBinding;
import javax.faces.el.MethodBinding;
import com.sun.facelets.util.FacesAPI;

/**
 * facelethandler for jMaki's 'ajax' component
 * 
 * this version is for JSF 1.2 on Java 5 (Tiger)
 *
 * @author Norbert von Truchsess
 */

public class AjaxWrapperTigerHandler extends ComponentHandler {

	private TagAttribute name;

	private TagAttribute service;

	private TagAttribute selected;

	private TagAttribute template;

	private TagAttribute script;

	private TagAttribute style;

	private TagAttribute args;

	private TagAttribute value;

	private TagAttribute valueChangeListener;
	
	private static final boolean supportsExpressions=(FacesAPI.getVersion()>=12);
	
	public AjaxWrapperTigerHandler(ComponentConfig config) {
		super(config);
		name = this.getRequiredAttribute("name");
		service = this.getAttribute("service");
		selected = this.getAttribute("selected");
		template = this.getAttribute("template");
		script = this.getAttribute("script");
		style = this.getAttribute("style");
		args = this.getAttribute("args");
		value = this.getAttribute("value");
		valueChangeListener = this.getAttribute("valueChangeListener");
	}


	protected void setAttributes(FaceletContext ctx, Object instance) {

		super.setAttributes(ctx, instance);
		AjaxWrapper component = (AjaxWrapper) instance;
		try {
			component.setName(this.name.getValue(ctx));
			if (this.service!=null)	{
				if (this.service.isLiteral()) {
					component.setService(this.service.getValue(ctx));
				} else {
					if (supportsExpressions) {
						component.setValueExpression("service",this.service.getValueExpression(ctx, String.class));
					} else {
						ValueBinding vb = ctx.getFacesContext().getApplication().createValueBinding(this.value.getValue(ctx));
						component.setValueBinding("service",vb);
					}
				}
			}
			if (this.style!=null) component.setStyle(this.style.getValue(ctx));
		    if (this.script!=null) component.setScript(this.script.getValue(ctx));
		    if (this.template!=null) component.setTemplate(this.template.getValue(ctx));
		    if (this.selected!=null) component.setSelected(this.selected.getValue(ctx));
		    if (this.args!=null) component.setArgs(this.args.getValue(ctx));
		    if (this.value!=null) {
		    	if (this.value.isLiteral()) {
		    		component.setValue(this.value.getValue(ctx));
		    	} else {
		    		if (supportsExpressions) {
		    			component.setValueExpression("value",this.value.getValueExpression(ctx,String.class));
		    		} else {
		    			ValueBinding vb = ctx.getFacesContext().getApplication().createValueBinding(this.value.getValue(ctx));
                        component.setValueBinding("value", vb);
		    		}
		    	}
		    }
		    if (this.valueChangeListener!=null) {
		    	Class args[] = { ValueChangeEvent.class };
		    	if (supportsExpressions) {
		    		component.addValueChangeListener(new MethodExpressionValueChangeListener(this.valueChangeListener.getMethodExpression(ctx, null,args)));
		    	} else {
	                MethodBinding vb = ctx.getFacesContext().getApplication().createMethodBinding(this.valueChangeListener.getValue(ctx), args);
                    component.setValueChangeListener(vb);
		    	}
		    }
		} catch (Exception e) {
            throw new javax.faces.FacesException("Unable to process attributes of jMaki component: " + component.getId(),e);
		}
	}
}