package jmaki.runtime.jsf;

import javax.servlet.ServletContext;
import javax.faces.context.FacesContext;
import jmaki.runtime.AjaxContext;
import jmaki.runtime.UriManager;
import jmaki.runtime.UriManagerFactory;
import jmaki.runtime.AjaxInitParameters;
import jmaki.runtime.jsf.AjaxWrapperPhaseListener;

/**
 * @author Norbert von Truchsess
 */
public class JsfUriManagerImpl implements UriManager,UriManagerFactory {

	public String buildClassPathResourceReference(AjaxContext ctx, String resource) {
		if (ctx.getIsExtensionMapped())
			return ctx.getApplicationRoot()+jsfRoot+resource;
		else
			return ctx.getServletRoot()+resource;
	}

	public String buildWebAppResourceReference(AjaxContext ctx, String resource) {
		return ctx.getApplicationRoot()+resource;
	}
	
	public String buildExternalResourceReference(AjaxContext ctx, String resource) {
		return externalRoot==null ? ctx.getWebRoot()+resource : ctx.getWebRoot()+externalRoot+resource;
	}
	
	public String buildDynamicResourceReference(AjaxContext ctx, String resource) {
		if (resource.startsWith("#{") && resource.endsWith("}")) {
			resource = resource.substring(2, resource.length()-1)+AjaxWrapperPhaseListener.JMAKI_AJAX;
		}
		return jsfRoot==null ? ctx.getServletRoot()+resource : ctx.getApplicationRoot()+jsfRoot+resource;
	}
	
	public UriManager getInstance(ServletContext ctx) { 
		if (!this.isInitialized) init(ctx); 
		return this;
	}
        
	private boolean isInitialized = false;

	private void init(ServletContext ctx) {
		if (ctx.getInitParameter(AjaxInitParameters.JSFROOT)!=null) {
			jsfRoot = ctx.getInitParameter(AjaxInitParameters.JSFROOT);
		} else
			jsfRoot = "/faces";
		if (ctx.getInitParameter(AjaxInitParameters.EXTERNALROOT)!=null) {
			externalRoot = ctx.getInitParameter(AjaxInitParameters.EXTERNALROOT);
		}
		this.isInitialized = true;
	}
	
	private String jsfRoot = null;
	private String externalRoot = null;
}
