package jmaki.runtime.jsf.shale;

import java.io.IOException;
import java.util.logging.Logger;

import javax.faces.context.FacesContext;
import javax.faces.el.EvaluationException;
import javax.faces.el.MethodBinding;
import javax.faces.el.ValueBinding;
import javax.faces.webapp.UIComponentTag;
import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import jmaki.runtime.jsf.AjaxResult;

import org.apache.shale.remoting.impl.*;

public class RemotingProcessor extends FilteringProcessor {

    private static Logger logger;
    
    public RemotingProcessor() {
    	super();
    }
	
    public static Logger getLogger() {
        if (logger == null) {
            logger = Logger.getLogger("jmaki.runtime.Log");
        }
        return logger;
    }     

    /**
     * <p>Convert the specified resource identifier into a method binding
     * expression, and delegate creation of the response to a call to the
     * identified method.  Call <code>FacesContext.responseComplete()</code>
     * to tell JavaServer Faces that the entire response has already been
     * created.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param resourceId Resource identifier used to select the appropriate response
     *  (this will generally be a context relative path starting with "/")
     *
     * @exception IllegalArgumentException if the view identifier is not
     *  well formed (starting with a '/' character)
     * @exception IOException if an input/output error occurs
     * @exception NullPointerException if <code>viewId</code> is <code>null</code>
     */
    public void process(FacesContext context, String resourceId) throws IOException {

        // If someone else has completed the response, we do not have
        // anything to do
        if (context.getResponseComplete()) {
            return;
        }

        // Filter based on our includes and excludes patterns
        if (!accept(resourceId)) {
        	getLogger().fine("Resource id '" + resourceId
                            + "' rejected by include/exclude rules");
            // Send an HTTP "not found" response to avoid giving the client
            // any information about a resource that exists and was refused,
            // versus a resource that does not exist
            sendNotFound(context, resourceId);
            context.responseComplete();
            return;
        }

        processAjaxRequest(context,resourceId);
        
        // Tell JavaServer Faces that the current response has been completed
        context.responseComplete();

    }

    private void processAjaxRequest(FacesContext context, String rootId) {

        HttpServletResponse response =
            (HttpServletResponse)context.getExternalContext().getResponse();

        Object object = context.getExternalContext().getRequest();

        if (!(object instanceof HttpServletRequest)) {
            // PortletRequest? Handle that here?
            return;
        }

        HttpServletRequest request = (HttpServletRequest)object;      
        
        String cmd = request.getParameter("cmd");
        // String uuid = request.getParameter("uuid");
        // String argString = request.getParameter("args");
        // String[] args = null;
        // if (argString != null) {
        //    args = argString.split(",");
        // }
        
        int targetIndex = rootId.lastIndexOf("/");
        // String[] nRoot = rootId.split("/");
        // for the case where we are not under an extension mapping
        if (targetIndex < 0) targetIndex = 0;
        String target = rootId.substring(targetIndex, rootId.length());
        String[] bindingArgs = target.split("-");
        String tO =  bindingArgs[0].substring(1,bindingArgs[0].length());
//        String tM = bindingArgs[1].substring(0,(bindingArgs[1].length() - ".ajax".length()));
        String tM = bindingArgs[1].substring(0,bindingArgs[1].length());
        String bindingExpression = "#{" + tO + "." + tM  + "}";
        try {

            if ((cmd != null) && "update".equals(cmd)) {
                String value = request.getParameter("value");
                if (value != null) {
                    boolean success = updateValueBoundObject(context,bindingExpression, value);
                    response.setHeader("Cache-Control", "no-cache");
                    response.getWriter().write("<status>" + success + "</status>");
                }
            } else {  
                AjaxResult result = invokeBinding(context, bindingExpression);
                if (result.getResponseType() == AjaxResult.XML) {
                    response.setContentType("text/xml");
                }
                response.setHeader("Cache-Control", "no-cache");
                response.getWriter().write(result.toString());
            }
        } catch (EvaluationException ee) {
            getLogger().severe(ee.toString());
        } catch (IOException ioe) {
            getLogger().severe(ioe.toString());
        } finally {
        }
        context.responseComplete();
        return;
    }
    
    private AjaxResult invokeBinding(FacesContext context, String methodBinding) {
        if (UIComponentTag.isValueReference(methodBinding)) {
            Class[] argTypes = { FacesContext.class, AjaxResult.class };
            MethodBinding vb = context.getApplication().createMethodBinding(methodBinding, argTypes);
            AjaxResult result = new AjaxResult();
            Object[] args = { context, result };
            vb.invoke(context, args);
            return result;
        } else {
            getLogger().severe("AjaxWrapperPhaseListener Error: Bad method binding: " + methodBinding);
        }
        return null;
    }
    
    private boolean updateValueBoundObject(FacesContext context, String valueBinding, String value) {
        try {
            ValueBinding vb = context.getApplication().createValueBinding(valueBinding);
            vb.setValue(context, value);
        } catch (EvaluationException ex) {
            return false;
        }
            return true;
    }
    
    /**
     * <p>Send a "not found" HTTP response, if possible.  Otherwise, throw an
     * <code>IllegalArgumentException</code> that will ripple out.</p>
     *
     * @param context <code>FacesContext</code> for the current request
     * @param resourceId Resource identifier of the resource that was not found
     *
     * @exception IllegalArgumentException if we cannot send an HTTP response
     * @exception IOException if an input/output error occurs
     *
     * @since 1.0.4
     */
    private void sendNotFound(FacesContext context, String resourceId) throws IOException {

        if (servletRequest(context)) {
            HttpServletResponse response = (HttpServletResponse)
              context.getExternalContext().getResponse();
            response.sendError(HttpServletResponse.SC_NOT_FOUND, resourceId);
        } else {
            throw new IllegalArgumentException(resourceId);
        }

    }

    /**
     * <p>Return <code>true</code> if we are processing a servlet request (as
     * opposed to a portlet request).</p>
     *
     * @param context <code>FacesContext</code> for the current request
     *
     * @since 1.0.4
     */
    private boolean servletRequest(FacesContext context) {

        return context.getExternalContext().getContext() instanceof ServletContext;

    }
}