package jmaki.runtime.jsf.shale;

import javax.servlet.ServletContext;

import jmaki.runtime.AjaxContext;
import jmaki.runtime.AjaxException;
import jmaki.runtime.UriManager;
import jmaki.runtime.UriManagerFactory;

import org.apache.shale.remoting.Mappings;
import org.apache.shale.remoting.Mapping;
import org.apache.shale.remoting.Mechanism;
import org.apache.shale.remoting.faces.MappingsHelper;
import javax.faces.context.FacesContext;

import java.util.Iterator;

public class RemotingUriManager implements UriManager, UriManagerFactory {
	
	private Mappings mappings = null;
	
	private Mapping classPathMapping = null;
	
	private Mapping dynamicMapping = null;
	
	private Mapping webappMapping = null;
	
	private Mapping externalMapping = null;

	public String buildClassPathResourceReference(AjaxContext ctx, String resource) throws AjaxException {
		if (classPathMapping==null) throw new AjaxException("unable to build ClassPathResourceReference: no mapping for "+Mechanism.CLASS_RESOURCE+" defined");
		return classPathMapping.mapResourceId(FacesContext.getCurrentInstance(),"/META-INF"+resource);
	}

	public String buildDynamicResourceReference(AjaxContext ctx, String resource) throws AjaxException {
		if (dynamicMapping==null) throw new AjaxException("unable to build DynamicResourceReference: no mapping for "+Mechanism.DYNAMIC_RESOURCE+" defined");
		if (resource.startsWith("#{") && resource.endsWith("}")) {
			resource = "/"+resource.substring(2,resource.length()-1).replace('.','-');
		}
		return dynamicMapping.mapResourceId(FacesContext.getCurrentInstance(),resource);
	}

	public String buildExternalResourceReference(AjaxContext ctx, String resource) throws AjaxException {
		if (externalMapping==null) throw new AjaxException("unable to build ExternalResourceReference: no mapping for "+Mechanism.OTHER_RESOURCE+" defined");
		return externalMapping.mapResourceId(FacesContext.getCurrentInstance(),resource);
	}

	public String buildWebAppResourceReference(AjaxContext ctx, String resource) throws AjaxException {
		if (webappMapping==null) throw new AjaxException("unable to build WebAppResourceReference: no mapping for "+Mechanism.WEBAPP_RESOURCE+" defined");
		return webappMapping.mapResourceId(FacesContext.getCurrentInstance(),resource);
	}
	
	public UriManager getInstance(ServletContext ctx) throws AjaxException {
		if ((mappings = new MappingsHelper().getMappings(FacesContext.getCurrentInstance()))==null) 
			throw new AjaxException("could not retrieve shale remoting mappings from application scope");
		Iterator mappingsIterator = mappings.getMappings().listIterator();
		while (mappingsIterator.hasNext()) {
			Mapping mapping = (Mapping) mappingsIterator.next();
			Mechanism mechanism = mapping.getMechanism();
			if (mechanism.equals(Mechanism.CLASS_RESOURCE))
				this.classPathMapping=mapping;
			if (mechanism.equals(Mechanism.DYNAMIC_RESOURCE))
				this.dynamicMapping=mapping;
			if (mechanism.equals(Mechanism.WEBAPP_RESOURCE))
				this.webappMapping=mapping;
			if (mechanism.equals(Mechanism.OTHER_RESOURCE))
				this.externalMapping=mapping;
		}
		return this;
	}
        public String getJSFRoot() {
            return null;
        }
        
        public void setJSFRoot(String root) {}
}
