/* Copyright 2004 Sun Microsystems, Inc.  All rights reserved.  You may not modify, use, reproduce, or distribute this software except in compliance with the terms of the License at: 
 http://bpcatalog.dev.java.net/LICENSE.txt
 $Id: AjaxTag.java,v 1.1 2007/04/10 04:22:41 gmurray71 Exp $ */

package jmaki.runtime.jsp;

import javax.servlet.jsp.*;
import javax.servlet.*;
import javax.servlet.http.*;
import javax.servlet.jsp.tagext.*;

import jmaki.runtime.AjaxCommonRenderer;
import jmaki.runtime.AjaxContext;
import jmaki.runtime.config.WidgetConfigException;
import jmaki.runtime.WidgetResourceException;

import java.util.logging.*;
import java.io.IOException;

/**
 * Base tag for AJAX Wrappers.
 *
 * @author Greg Murray
 */
public class AjaxTag extends BodyTagSupport {
 
    private static Logger logger;
    
    private boolean embedded = false;
    
    private String id = null;
    private String onLoad = null;
    private String args = null;
    private String name = null;
    private String style = null;
    private String script = null;
    private String template = null;
    private String service = null;
    private String type = null;
    private String value = null;
    private String selected = null;
    private ServletContext ctx = null;
    
    public void setId(String id) {
        this.id=id;
    }
    
    public void setName(String name) {
        this.name=name;
    }
    
    public void setArgs(String args) {
        this.args=args;
    }
    
    public void setService(String service) {
        this.service=service;
    }
    
    public void setStyle(String style) {
        this.style=style;
    }
    
    public void setScript(String script) {
        this.script=script;
    }
    
    public void setTemplate(String template) {
        this.template=template;
    }
    
    public void setSelected(String selected) {
        this.selected=selected;
    }
    
    public void setValue(String value) {
        this.value=value;
    }
    
    public int doStartTag() throws JspTagException {
        return EVAL_BODY_BUFFERED;
    }
    
    public int doAfterBody() throws JspTagException {
        return SKIP_BODY; 
    }
    
    public int doEndTag() throws JspTagException {
        PageContext pctx = (PageContext)pageContext;
        HttpServletRequest request = (HttpServletRequest)pctx.getRequest();
        HttpServletResponse response = (HttpServletResponse)pctx.getResponse();
        if (ctx == null) ctx = pctx.getServletContext();
        JspWriter out = pageContext.getOut();
        // now that we have everything do the evaluation
        try {
            AjaxContext ajx = AjaxContext.getInstance(ctx, request, response, out);
            AjaxCommonRenderer common = new AjaxCommonRenderer(ajx);
            common.setUuid(this.id);
            common.setArgs(this.args);
            common.setName(this.name);
            common.setScript(this.script);
            common.setTemplate(this.template);
            common.setService(this.service);
            common.setValue(this.value);
            common.setSelected(this.selected);
            common.setStyle(this.style);
            common.setType(this.type);
            common.doProcess();
        } catch (WidgetResourceException wre) {
            try {
                out.println("Error loading widget resource for " + this.name + ":" + wre.getMessage());
            } catch (IOException iox) {
                // swallow error
            }            
        } catch (WidgetConfigException wfg) {
            try {
                out.println("Error loading widget " + this.name + ":" + wfg.getMessage());
            } catch (IOException iox) {
                // swallow error
            }
        } catch (IOException ioe) {
            getLogger().severe(ioe.getMessage());
            //swallow error
        } finally {
            reset();
        }
        return EVAL_PAGE;
    }
    
    private void reset() {
        script = null;
        style = null;
        service = null;
        name = null;
        embedded = false;
        template = null;
        type = null;
        onLoad = null;
        args = null;
        id = null;
    }
    
    public static Logger getLogger() {
        if (logger == null) {
            logger = Logger.getLogger("jmaki.runtime.Log");
        }
        return logger;
    }

}

