package com.sun.jmaki.chat;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.util.*;


public class ChatServlet extends HttpServlet implements Runnable {
    
    private ServletContext context;
    private Thread timer;
    
    private long lastPurge = 0;
    private int CHAT_MAX = 10;
    private int SLEEP = 5000;
    private long timeout;
    private Hashtable users;
    private Vector messages;
    private int counter = 0;
       
    public void init(ServletConfig config) throws ServletException {
	    super.init(config);
        timeout = (new Long(CHAT_MAX * SLEEP)).longValue();;
        lastPurge  = System.currentTimeMillis();
        System.out.println("ChatServlet 1.1: initializing...");
        users = new Hashtable();
        users.put("Server", new Chatter("Server", "", 0));
        messages = new Vector();
        timer = new Thread(this);
        timer.setPriority(Thread.MIN_PRIORITY);
        timer.start();
    }

    public void run() {
        while (true) {
            try {
                timer.sleep(SLEEP);
            } catch (InterruptedException ie) {
            }
            if (counter == CHAT_MAX ) {
                // check the users that ar in active        
                ArrayList rUsers = null;  
                Iterator it = users.keySet().iterator();

                while (it.hasNext()) {
                    String userId = (String)it.next();
                    Chatter c = (Chatter)users.get(userId);
                    if (!userId.equals("Server") && (c.getLastAccess() - lastPurge) < timeout) {
                        if (rUsers == null) {
                            rUsers = new ArrayList();
                        }
                        rUsers.add(userId);
                    } else {
                       c.resetIndex();
                    }
                }
                lastPurge = System.currentTimeMillis();
                messages.removeAllElements();
                if (rUsers != null) {
                    Iterator it2 = rUsers.iterator();
                    while (it2.hasNext()) {
                        String userId = (String)it2.next();
                        users.remove(userId);
                    }
                }
                counter = 0;
            } else {
                counter++;
            }
             
        }
    }
   
    public void doGet(HttpServletRequest request, HttpServletResponse  response)
        throws IOException, ServletException {
        doProcess(request, response);
    }

    public void doPost(HttpServletRequest request, HttpServletResponse  response)
        throws IOException, ServletException {
        doProcess(request, response);
        
    }
    
    private void doProcess(HttpServletRequest request, HttpServletResponse response)
        throws IOException, ServletException {
            try {
                String action = request.getParameter("action");
                // negotiate a userid
                if ("valid-register".equals(action)) {
                    String userId = request.getParameter("userid").trim();
                    // content-type must be set to text/xml
                    response.setContentType("text/xml");
                    // needed to keep some browsers from using locally cached data
                    response.setHeader("Cache-Control", "no-cache");
                    PrintWriter pw = response.getWriter();
                    if (users.containsKey(userId)) {
                        pw.write("<message>invalid</message>");
                    } else {
                        pw.write("<message>valid</message>");
                    }
                    pw.flush(); 
                } else if ("register".equals(action)) {
                    String userId = request.getParameter("userid").trim();
                    String iconURI = request.getParameter("icon");
                    // content-type must be set to text/xml
                    response.setContentType("text/xml");
                    // needed to keep some browsers from using locally cached data
                    response.setHeader("Cache-Control", "no-cache");
                    PrintWriter pw = response.getWriter();
                    if (users.containsKey(userId)) {
                        pw.write("<message>invalid</message>");
                    } else {
                        // add the selected icon id later
                        System.out.println("ChatServlet adding user:" + userId);
                        Chatter c = new Chatter(userId,iconURI, System.currentTimeMillis());
                        c.setIndex(messages.size());
                        users.put(userId, c );
                        pw.write("<message>success</message>");
                        messages.add(new Message("Server", userId + " has joined."));
                    }
                } else if ("add-message".equals(action)) {
                    String userId = request.getParameter("userid");
                    if (userId != null) userId = userId.trim();
                    String message = request.getParameter("message");
                    
                    // content-type must be set to text/xml
                    response.setContentType("text/xml");
                    
                    // needed to keep some browsers from using locally cached data
                    response.setHeader("Cache-Control", "no-cache");
                    PrintWriter pw = response.getWriter();
                    if (users.containsKey(userId)) {
                        Chatter c = (Chatter)users.get(userId);
                        c.setLastAccess(System.currentTimeMillis());
                        messages.add(new Message(userId, message));
                        pw.write("<message>valid</message>");
                    } else {
                        response.setStatus(HttpServletResponse.SC_FORBIDDEN);
                        pw.write("<message>forbidden</message>");
                    }                
                    pw.flush();
                } else if ("signoff".equals(action)) {
                    String userId = request.getParameter("userid");
                    if (userId != null) userId = userId.trim();
                    if (users.containsKey(userId)) {
                        users.remove(userId);
                        messages.add(new Message("Server", userId + " has signed off."));
                        // content-type must be set to text/xml
                       response.setContentType("text/xml");
                       PrintWriter pw = response.getWriter();
                       pw.write("<message>signed off</message>");
                    }
                } else if ("get-messages".equals(action)) {
                    String userId = request.getParameter("userid");
                    if (userId != null) userId = userId.trim();
 
                    // content-type must be set to text/xml
                    response.setContentType("text/xml");              
                    // needed to keep some browsers from using locally cached data
                    response.setHeader("Cache-Control", "no-cache");
                    PrintWriter pw = response.getWriter();
                    if (users.containsKey(userId)) {
                        Chatter c = (Chatter)users.get(userId);
                        c.setLastAccess(System.currentTimeMillis());
                        int start = c.getIndex();                     
                        if (start < messages.size()) {
                            c.setIndex(messages.size());
                            pw.write("<messages>");
                            for (int loop=start; loop < messages.size(); loop++) {
                                Message m = (Message)messages.get(loop);
                                pw.write("<message>");
                                pw.write(" <text>" + m.getMessage() + "</text>");   	
                                pw.write(" <sender>" + m.getSender() + "</sender>");
                                pw.write(" <icon>" + ((Chatter)users.get(m.getSender())).getIconURI() + "</icon>");
                                pw.write(" <style></style>");
                                pw.write("</message>");
                            }
                            pw.write("</messages>");
                        } else {
                              response.setStatus(HttpServletResponse.SC_NO_CONTENT);
                        }
                    } else {
                        System.out.println("User " + userId + " is forbidden.");
                        response.setStatus(HttpServletResponse.SC_FORBIDDEN);
                        pw.write("<message>forbidden</message>");
                    }
                    pw.flush();
                }
            } catch (Throwable e) {
                System.out.println("ChatServlet error:" + e);
                e.printStackTrace();
            }
        }
    
    public void destroy(){
        timer.destroy();
    }
    
    class Chatter {
        
      private int index = 0;
      private String id;
      private String icon;
      private long lastAccess = 0;
      
      public Chatter(String id, String icon, long last) {
          this.id = id;
          this.icon = icon;
          this.lastAccess = last;
      }
      
      public String getId() {
          return id;	
      }
      
      public int getIndex() {
          return index;
      }
      
      public void setIndex(int nIndex) {
          this.index = nIndex;
      }
      
      public void resetIndex() {
          index = 0;
      }
      
      public String getIconURI() {
          return icon;	
      }
      
      public void setLastAccess(long last) {
          lastAccess = last;
      }
      
      public long getLastAccess() {
          return lastAccess;
      }
    }
    
    class Message {
        
      private String sender;
      private String message;
      
      public Message(String sender, String message) {
          this.sender = sender;
          this.message = message;
      }
      
      public String getMessage(){
          return message;	
      }
      
      public String getSender(){
        return sender;
      }
    }
    
}

