/* Copyright 2006 Sun Microsystems, Inc.  All rights reserved.  You may not modify, use, reproduce, or distribute this software except in compliance with the terms of the License at: 
 http://developer.sun.com/berkeley_license.html
 $Id: RssServlet.java,v 1.6 2007/02/10 04:16:21 gmurray71 Exp $ 
*/

package com.sun.jmaki.rss;

import java.io.*;
import java.util.*;
import java.util.logging.*;

import javax.servlet.*;
import javax.servlet.http.*;
import javax.xml.transform.*;
import javax.xml.transform.stream.*;

import com.sun.jmaki.services.XmlHttpProxy;


public class RssServlet extends HttpServlet {
	
    private static String CACHE = "jmaki-rss-cache";
    private static boolean rDebug = false;
    private Logger logger = null;
    private ServletContext ctx;
    private XmlHttpProxy xhp;
    private String encoding = null;
    private long lastLoaded = 0;
    // default to reload once every 3 hours
    private long reloadInterval = 180000;

    
    /** Creates a new instance of RssServlet
     * @author Yutaka Yoshida
     * @author Greg Murray
     */
    public RssServlet() {
        if (rDebug) {
            logger = logger.getLogger("rssFeed");
        }
    }
    
    public void init(ServletConfig config) throws ServletException {
	    super.init(config);
        ctx = config.getServletContext();
        // if there is a proxyHost and proxyPort specified create an HttpClient with the proxy
        String proxyHost = ctx.getInitParameter("proxyHost");
        String proxyPortString = ctx.getInitParameter("proxyPort");
        encoding = ctx.getInitParameter("encoding");
        if (proxyHost != null && proxyPortString != null) {
            int proxyPort = 8080;
            try {
                proxyPort= new Integer(proxyPortString).intValue();
                xhp = new XmlHttpProxy(proxyHost, proxyPort);
            } catch (NumberFormatException nfe) {
                getLogger().severe("RssServlet: intialization error. The proxyPort must be a number");
                throw new ServletException("RssServlet: intialization error. The proxyPort must be a number");
            }
        } else {
            xhp = new XmlHttpProxy();
        }
        String reloadIntervalString = ctx.getInitParameter("reload-interval");
        if (reloadIntervalString != null) {
            try {
                reloadInterval= new Long(reloadIntervalString).longValue();
            } catch (NumberFormatException nfe) {
               getLogger().severe("RssServlet: intialization error. The reload interval must be a number"); 
            }
        }
	}
    
    public Logger getLogger() {
        if (logger == null) {
            logger = Logger.getLogger("com.sun.jmaki.Log");
        }
        return logger;
    }
    private void logMessage(String message) {
        if (rDebug) {
            getLogger().info(message);
        }
    }
   
    public void doGet(HttpServletRequest req, HttpServletResponse res) {		
		try {
			OutputStream out = res.getOutputStream();
			String urlString = req.getParameter("url");
            if (urlString == null) {
                out.write("RSS Servlet Error: 'url' parameter to RSS feed required".getBytes());
                out.flush();
                return;
            }
			String itemNumber = req.getParameter("itemNumber");
			String format = req.getParameter("format");
			getRssFeed(urlString, format, itemNumber, out, ctx);
		} catch (java.io.IOException iox) {
			getLogger().severe("RssServlet: caught " + iox);
		}
	}
    
    public void getRssFeed(String urlString,
    					   String style,
    					   String itemNumber,
    					   OutputStream out,
                           ServletContext context) throws IOException {
                               
           // first check the cach for the 
           Hashtable cache = (Hashtable)context.getAttribute(CACHE);
           if (cache == null) {
               cache = new Hashtable();
               context.setAttribute(CACHE, cache);
           }

           boolean reloadNeeded = false;
           long currentTime = System.currentTimeMillis();

           if ((currentTime - lastLoaded) > reloadInterval) {
               reloadNeeded = true;
           }
           
           String entry = null;
           
           // if the reload interval has not been reached return from the cache
           if (cache.containsKey(urlString) && !reloadNeeded) {
               entry = (String)cache.get(urlString);
           } else {

               if (style == null) {
                   style = "html";
               }
               
               if (itemNumber == null) {
                   itemNumber = "0";
               }
               
               Map paramMap = new HashMap();
               if (style != null) {
                   paramMap.put("style", style);
               }
               if (itemNumber != null) {
                   paramMap.put("number", itemNumber);
               }
               InputStream xslIS = RssServlet.class.getClassLoader().getResourceAsStream("META-INF/resources/rss.xsl");
               ByteArrayOutputStream bos = new ByteArrayOutputStream();
               xhp.doGet(urlString, bos, xslIS, paramMap);
               if (encoding == null) {
                   entry = bos.toString();
               } else {
                   entry = bos.toString(encoding);
               }
               cache.put(urlString, entry);
               lastLoaded = System.currentTimeMillis();
           }
           if (entry != null) {
            out.write(entry.getBytes());
           } else {
              getLogger().severe("RssServlet: " + urlString + " could not be loaded.");
           }
    }
}
